@@ -1,4 +1,4 @@
-import typing
+from typing import List, Optional, Tuple
 
 import numpy as np
 from pyrfr import regression
@@ -16,9 +16,49 @@ __version__ = "0.0.1"
 
 
 class RandomForestWithInstances(BaseModel):
-
     """Random forest that takes instance features into account.
 
+    Parameters
+    ----------
+    types : List[int]
+        Specifies the number of categorical values of an input dimension where
+        the i-th entry corresponds to the i-th input dimension. Let's say we
+        have 2 dimension where the first dimension consists of 3 different
+        categorical choices and the second dimension is continuous than we
+        have to pass [3, 0]. Note that we count starting from 0.
+    bounds : List[Tuple[float, float]]
+        bounds of input dimensions: (lower, uppper) for continuous dims; (n_cat, np.nan) for categorical dims
+    seed : int
+        The seed that is passed to the random_forest_run library.
+    log_y: bool
+        y values (passed to this RF) are expected to be log(y) transformed;
+        this will be considered during predicting
+    num_trees : int
+        The number of trees in the random forest.
+    do_bootstrapping : bool
+        Turns on / off bootstrapping in the random forest.
+    n_points_per_tree : int
+        Number of points per tree. If <= 0 X.shape[0] will be used
+        in _train(X, y) instead
+    ratio_features : float
+        The ratio of features that are considered for splitting.
+    min_samples_split : int
+        The minimum number of data points to perform a split.
+    min_samples_leaf : int
+        The minimum number of data points in a leaf.
+    max_depth : int
+        The maximum depth of a single tree.
+    eps_purity : float
+        The minimum difference between two target values to be considered
+        different
+    max_num_nodes : int
+        The maxmimum total number of nodes in a tree
+    instance_features : np.ndarray (I, K)
+        Contains the K dimensional instance features of the I different instances
+    pca_components : float
+        Number of components to keep when using PCA to reduce dimensionality of instance features. Requires to
+        set n_feats (> pca_dims).
+
     Attributes
     ----------
     rf_opts : regression.rf_opts
@@ -39,64 +79,22 @@ class RandomForestWithInstances(BaseModel):
     def __init__(
         self,
         configspace: ConfigurationSpace,
-        types: typing.List[int],
-        bounds: typing.List[typing.Tuple[float, float]],
+        types: List[int],
+        bounds: List[Tuple[float, float]],
         seed: int,
         log_y: bool = False,
         num_trees: int = N_TREES,
         do_bootstrapping: bool = True,
         n_points_per_tree: int = -1,
-        ratio_features: float = 5. / 6.,
+        ratio_features: float = 5.0 / 6.0,
         min_samples_split: int = 3,
         min_samples_leaf: int = 3,
         max_depth: int = 2**20,
         eps_purity: float = 1e-8,
         max_num_nodes: int = 2**20,
-        instance_features: typing.Optional[np.ndarray] = None,
-        pca_components: typing.Optional[int] = None,
+        instance_features: Optional[np.ndarray] = None,
+        pca_components: Optional[int] = None,
     ) -> None:
-        """
-        Parameters
-        ----------
-        types : List[int]
-            Specifies the number of categorical values of an input dimension where
-            the i-th entry corresponds to the i-th input dimension. Let's say we
-            have 2 dimension where the first dimension consists of 3 different
-            categorical choices and the second dimension is continuous than we
-            have to pass [3, 0]. Note that we count starting from 0.
-        bounds : List[Tuple[float, float]]
-            bounds of input dimensions: (lower, uppper) for continuous dims; (n_cat, np.nan) for categorical dims
-        seed : int
-            The seed that is passed to the random_forest_run library.
-        log_y: bool
-            y values (passed to this RF) are expected to be log(y) transformed;
-            this will be considered during predicting
-        num_trees : int
-            The number of trees in the random forest.
-        do_bootstrapping : bool
-            Turns on / off bootstrapping in the random forest.
-        n_points_per_tree : int
-            Number of points per tree. If <= 0 X.shape[0] will be used
-            in _train(X, y) instead
-        ratio_features : float
-            The ratio of features that are considered for splitting.
-        min_samples_split : int
-            The minimum number of data points to perform a split.
-        min_samples_leaf : int
-            The minimum number of data points in a leaf.
-        max_depth : int
-            The maximum depth of a single tree.
-        eps_purity : float
-            The minimum difference between two target values to be considered
-            different
-        max_num_nodes : int
-            The maxmimum total number of nodes in a tree
-        instance_features : np.ndarray (I, K)
-            Contains the K dimensional instance features of the I different instances
-        pca_components : float
-            Number of components to keep when using PCA to reduce dimensionality of instance features. Requires to
-            set n_feats (> pca_dims).
-        """
         super().__init__(
             configspace=configspace,
             types=types,
@@ -112,8 +110,7 @@ class RandomForestWithInstances(BaseModel):
         self.rf_opts = regression.forest_opts()
         self.rf_opts.num_trees = num_trees
         self.rf_opts.do_bootstrapping = do_bootstrapping
-        max_features = 0 if ratio_features > 1.0 else \
-            max(1, int(len(types) * ratio_features))
+        max_features = 0 if ratio_features > 1.0 else max(1, int(len(types) * ratio_features))
         self.rf_opts.tree_opts.max_features = max_features
         self.rf_opts.tree_opts.min_samples_to_split = min_samples_split
         self.rf_opts.tree_opts.min_samples_in_leaf = min_samples_leaf
@@ -126,11 +123,20 @@ class RandomForestWithInstances(BaseModel):
         self.rf = None  # type: regression.binary_rss_forest
 
         # This list well be read out by save_iteration() in the solver
-        self.hypers = [num_trees, max_num_nodes, do_bootstrapping,
-                       n_points_per_tree, ratio_features, min_samples_split,
-                       min_samples_leaf, max_depth, eps_purity, self.seed]
+        self.hypers = [
+            num_trees,
+            max_num_nodes,
+            do_bootstrapping,
+            n_points_per_tree,
+            ratio_features,
+            min_samples_split,
+            min_samples_leaf,
+            max_depth,
+            eps_purity,
+            self.seed,
+        ]
 
-    def _train(self, X: np.ndarray, y: np.ndarray) -> 'RandomForestWithInstances':
+    def _train(self, X: np.ndarray, y: np.ndarray) -> "RandomForestWithInstances":
         """Trains the random forest on X and y.
 
         Parameters
@@ -159,8 +165,8 @@ class RandomForestWithInstances(BaseModel):
         return self
 
     def _init_data_container(self, X: np.ndarray, y: np.ndarray) -> regression.default_data_container:
-        """Fills a pyrfr default data container, s.t. the forest knows
-        categoricals and bounds for continous data
+        """Fills a pyrfr default data container, s.t. the forest knows categoricals and bounds for
+        continous data.
 
         Parameters
         ----------
@@ -187,16 +193,14 @@ class RandomForestWithInstances(BaseModel):
             data.add_data_point(row_X, row_y)
         return data
 
-    def _predict(self, X: np.ndarray,
-                 cov_return_type: typing.Optional[str] = 'diagonal_cov') \
-            -> typing.Tuple[np.ndarray, np.ndarray]:
+    def _predict(self, X: np.ndarray, cov_return_type: Optional[str] = "diagonal_cov") -> Tuple[np.ndarray, np.ndarray]:
         """Predict means and variances for given X.
 
         Parameters
         ----------
         X : np.ndarray of shape = [n_samples,
                                    n_features (config + instance features)]
-        cov_return_type: typing.Optional[str]
+        cov_return_type: Optional[str]
             Specifies what to return along with the mean. Refer ``predict()`` for more information.
 
         Returns
@@ -207,11 +211,10 @@ class RandomForestWithInstances(BaseModel):
             Predictive variance
         """
         if len(X.shape) != 2:
-            raise ValueError(
-                'Expected 2d array, got %dd array!' % len(X.shape))
+            raise ValueError("Expected 2d array, got %dd array!" % len(X.shape))
         if X.shape[1] != len(self.types):
-            raise ValueError('Rows in X should have %d entries but have %d!' % (len(self.types), X.shape[1]))
-        if cov_return_type != 'diagonal_cov':
+            raise ValueError("Rows in X should have %d entries but have %d!" % (len(self.types), X.shape[1]))
+        if cov_return_type != "diagonal_cov":
             raise ValueError("'cov_return_type' can only take 'diagonal_cov' for this model")
 
         X = self._impute_inactive(X)
@@ -231,7 +234,7 @@ class RandomForestWithInstances(BaseModel):
             preds_as_array = np.zeros((X.shape[0], self.rf_opts.num_trees, third_dimension)) * np.NaN
             for i, preds_per_tree in enumerate(all_preds):
                 for j, pred in enumerate(preds_per_tree):
-                    preds_as_array[i, j, :len(pred)] = pred
+                    preds_as_array[i, j, : len(pred)] = pred
 
             # Do all necessary computation with vectorized functions
             preds_as_array = np.log(np.nanmean(np.exp(preds_as_array), axis=2) + VERY_SMALL_NUMBER)
@@ -251,7 +254,7 @@ class RandomForestWithInstances(BaseModel):
 
         return means.reshape((-1, 1)), vars_.reshape((-1, 1))
 
-    def predict_marginalized_over_instances(self, X: np.ndarray) -> typing.Tuple[np.ndarray, np.ndarray]:
+    def predict_marginalized_over_instances(self, X: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
         """Predict mean and variance marginalized over all instances.
 
         Returns the predictive mean and variance marginalised over all
@@ -277,9 +280,7 @@ class RandomForestWithInstances(BaseModel):
         vars : np.ndarray  of shape = [n_samples, 1]
             Predictive variance
         """
-
-        if self.instance_features is None or \
-                len(self.instance_features) == 0:
+        if self.instance_features is None or len(self.instance_features) == 0:
             mean_, var = self.predict(X)
             assert var is not None  # please mypy
 
@@ -288,12 +289,9 @@ class RandomForestWithInstances(BaseModel):
             return mean_, var
 
         if len(X.shape) != 2:
-            raise ValueError(
-                'Expected 2d array, got %dd array!' % len(X.shape))
+            raise ValueError("Expected 2d array, got %dd array!" % len(X.shape))
         if X.shape[1] != len(self.bounds):
-            raise ValueError('Rows in X should have %d entries but have %d!' %
-                             (len(self.bounds),
-                              X.shape[1]))
+            raise ValueError("Rows in X should have %d entries but have %d!" % (len(self.bounds), X.shape[1]))
 
         X = self._impute_inactive(X)
 
@@ -302,7 +300,7 @@ class RandomForestWithInstances(BaseModel):
 
             # marginalize over instances
             # 1. get all leaf values for each tree
-            preds_trees = [[] for i in range(self.rf_opts.num_trees)]  # type: typing.List[typing.List[float]]
+            preds_trees = [[] for i in range(self.rf_opts.num_trees)]  # type: List[List[float]]
 
             for feat in self.instance_features:
                 x_ = np.concatenate([x, feat])
@@ -313,8 +311,7 @@ class RandomForestWithInstances(BaseModel):
             # 2. average in each tree
             if self.log_y:
                 for tree_id in range(self.rf_opts.num_trees):
-                    dat_[i, tree_id] = \
-                        np.log(np.exp(np.array(preds_trees[tree_id])).mean())
+                    dat_[i, tree_id] = np.log(np.exp(np.array(preds_trees[tree_id])).mean())
             else:
                 for tree_id in range(self.rf_opts.num_trees):
                     dat_[i, tree_id] = np.array(preds_trees[tree_id]).mean()
@@ -323,6 +320,9 @@ class RandomForestWithInstances(BaseModel):
         mean_ = dat_.mean(axis=1)
         var = dat_.var(axis=1)
 
+        if var is None:
+            raise RuntimeError("The variance must not be none.")
+
         var[var < self.var_threshold] = self.var_threshold
 
         if len(mean_.shape) == 1:
