@@ -1,17 +1,16 @@
+from typing import Any, Dict, List, Optional, Tuple
+
 import numpy as np
 
 from smac.configspace import ConfigurationSpace
 from smac.epm.base_epm import AbstractEPM
 from smac.epm.rf_with_instances import RandomForestWithInstances
 
-from typing import List, Dict, Any, Optional, Tuple
-
 __copyright__ = "Copyright 2021, AutoML.org Freiburg-Hannover"
 __license__ = "3-clause BSD"
 
 
 class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
-
     """Wrapper for the random forest to predict multiple targets.
 
     Only the a list with the target names and the types array for the
@@ -19,6 +18,27 @@ class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
     the random forest can be passed via kwargs. Consult the documentation of
     the random forest for the hyperparameters and their meanings.
 
+
+    Parameters
+    ----------
+    target_names : list
+        List of str, each entry is the name of one target dimension. Length
+        of the list will be ``n_objectives``.
+    types : List[int]
+        Specifies the number of categorical values of an input dimension where
+        the i-th entry corresponds to the i-th input dimension. Let's say we
+        have 2 dimension where the first dimension consists of 3 different
+        categorical choices and the second dimension is continuous than we
+        have to pass [3, 0]. Note that we count starting from 0.
+    bounds : List[Tuple[float, float]]
+        bounds of input dimensions: (lower, uppper) for continuous dims; (n_cat, np.nan) for categorical dims
+    instance_features : np.ndarray (I, K)
+        Contains the K dimensional instance features of the I different instances
+    pca_components : float
+        Number of components to keep when using PCA to reduce dimensionality of instance features. Requires to
+        set n_feats (> pca_dims).
+
+
     Attributes
     ----------
     target_names
@@ -37,27 +57,6 @@ class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
         instance_features: Optional[np.ndarray] = None,
         pca_components: Optional[int] = None,
     ) -> None:
-        """Constructor
-
-        Parameters
-        ----------
-        target_names : list
-            List of str, each entry is the name of one target dimension. Length
-            of the list will be ``n_objectives``.
-        types : List[int]
-            Specifies the number of categorical values of an input dimension where
-            the i-th entry corresponds to the i-th input dimension. Let's say we
-            have 2 dimension where the first dimension consists of 3 different
-            categorical choices and the second dimension is continuous than we
-            have to pass [3, 0]. Note that we count starting from 0.
-        bounds : List[Tuple[float, float]]
-            bounds of input dimensions: (lower, uppper) for continuous dims; (n_cat, np.nan) for categorical dims
-        instance_features : np.ndarray (I, K)
-            Contains the K dimensional instance features of the I different instances
-        pca_components : float
-            Number of components to keep when using PCA to reduce dimensionality of instance features. Requires to
-            set n_feats (> pca_dims).
-        """
         super().__init__(
             configspace=configspace,
             bounds=bounds,
@@ -72,10 +71,11 @@ class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
         self.target_names = target_names
         self.num_targets = len(self.target_names)
         print(seed, rf_kwargs)
-        self.estimators = [RandomForestWithInstances(configspace, types, bounds, **rf_kwargs)
-                           for _ in range(self.num_targets)]
+        self.estimators = [
+            RandomForestWithInstances(configspace, types, bounds, **rf_kwargs) for _ in range(self.num_targets)
+        ]
 
-    def _train(self, X: np.ndarray, Y: np.ndarray) -> 'UncorrelatedMultiObjectiveRandomForestWithInstances':
+    def _train(self, X: np.ndarray, Y: np.ndarray) -> "UncorrelatedMultiObjectiveRandomForestWithInstances":
         """Trains the random forest on X and y.
 
         Parameters
@@ -95,9 +95,7 @@ class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
 
         return self
 
-    def _predict(self, X: np.ndarray,
-                 cov_return_type: Optional[str] = 'diagonal_cov') \
-            -> Tuple[np.ndarray, np.ndarray]:
+    def _predict(self, X: np.ndarray, cov_return_type: Optional[str] = "diagonal_cov") -> Tuple[np.ndarray, np.ndarray]:
         """Predict means and variances for given X.
 
         Parameters
@@ -114,7 +112,7 @@ class UncorrelatedMultiObjectiveRandomForestWithInstances(AbstractEPM):
         vars : np.ndarray  of shape = [n_samples, n_objectives]
             Predictive variance
         """
-        if cov_return_type != 'diagonal_cov':
+        if cov_return_type != "diagonal_cov":
             raise ValueError("'cov_return_type' can only take 'diagonal_cov' for this model")
 
         mean = np.zeros((X.shape[0], self.num_targets))
