@@ -10,8 +10,7 @@ __license__ = "3-clause BSD"
 
 
 class DefaultConfiguration(InitialDesign):
-
-    """Initial design that evaluates default configuration"""
+    """Initial design that evaluates default configuration."""
 
     def _select_configurations(self) -> List[Configuration]:
         """Selects the default configuration.
@@ -21,7 +20,6 @@ class DefaultConfiguration(InitialDesign):
         config: Configuration
             Initial incumbent configuration.
         """
-
         config = self.cs.get_default_configuration()
-        config.origin = 'Default'
+        config.origin = "Default"
         return [config]
