@@ -1,9 +1,8 @@
 import typing
 
-from scipy.stats.qmc import LatinHypercube
-
 from ConfigSpace.configuration_space import Configuration
 from ConfigSpace.hyperparameters import Constant
+from scipy.stats.qmc import LatinHypercube
 
 from smac.initial_design.initial_design import InitialDesign
 
@@ -13,7 +12,7 @@ __license__ = "3-clause BSD"
 
 
 class LHDesign(InitialDesign):
-    """Latin Hypercube design
+    """Latin Hypercube design.
 
     Attributes
     ----------
@@ -24,14 +23,13 @@ class LHDesign(InitialDesign):
     """
 
     def _select_configurations(self) -> typing.List[Configuration]:
-        """Selects a single configuration to run
+        """Selects a single configuration to run.
 
         Returns
         -------
         config: Configuration
             initial incumbent configuration
         """
-
         params = self.cs.get_hyperparameters()
 
         constants = 0
@@ -41,6 +39,4 @@ class LHDesign(InitialDesign):
 
         lhd = LatinHypercube(d=len(params) - constants, seed=self.rng.randint(0, 1000000)).random(n=self.init_budget)
 
-        return self._transform_continuous_designs(design=lhd,
-                                                  origin='LHD',
-                                                  cs=self.cs)
+        return self._transform_continuous_designs(design=lhd, origin="LHD", cs=self.cs)
