@@ -1,16 +1,21 @@
-import logging
 import typing
 
+import logging
+
 import numpy as np
 
+from smac.configspace import Configuration
 from smac.intensification.abstract_racer import AbstractRacer, RunInfoIntent
+from smac.intensification.parallel_scheduling import ParallelScheduler
 from smac.intensification.successive_halving import _SuccessiveHalving
 from smac.optimizer.epm_configuration_chooser import EPMChooser
-from smac.intensification.parallel_scheduling import ParallelScheduler
+from smac.runhistory.runhistory import (  # noqa: F401
+    RunHistory,
+    RunInfo,
+    RunValue,
+    StatusType,
+)
 from smac.stats.stats import Stats
-from smac.configspace import Configuration
-from smac.runhistory.runhistory import RunHistory
-from smac.runhistory.runhistory import RunValue, RunInfo, StatusType  # noqa: F401
 from smac.utils.io.traj_logging import TrajLogger
 
 __author__ = "Ashwin Raaghav Narayanan"
@@ -19,7 +24,7 @@ __license__ = "3-clause BSD"
 
 
 class _Hyperband(_SuccessiveHalving):
-    """ Races multiple challengers against an incumbent using Hyperband method
+    """Races multiple challengers against an incumbent using Hyperband method.
 
     This class contains the logic to implement:
     "BOHB: Robust and Efficient Hyperparameter Optimization at Scale" (Falkner et al. 2018)
@@ -38,7 +43,7 @@ class _Hyperband(_SuccessiveHalving):
     rng : np.random.RandomState
     instances : typing.List[str]
         list of all instance ids
-    instance_specifics : typing.Mapping[str,np.ndarray]
+    instance_specifics : typing.Mapping[str, str]
         mapping from instance name to instance specific string
     cutoff : typing.Optional[int]
         runtime cutoff of TA runs
@@ -74,66 +79,68 @@ class _Hyperband(_SuccessiveHalving):
         Allows to identify the _Hyperband instance in case of multiple ones
     """
 
-    def __init__(self,
-                 stats: Stats,
-                 traj_logger: TrajLogger,
-                 rng: np.random.RandomState,
-                 instances: typing.List[str],
-                 instance_specifics: typing.Mapping[str, np.ndarray] = None,
-                 cutoff: typing.Optional[float] = None,
-                 deterministic: bool = False,
-                 initial_budget: typing.Optional[float] = None,
-                 max_budget: typing.Optional[float] = None,
-                 eta: float = 3,
-                 run_obj_time: bool = True,
-                 n_seeds: typing.Optional[int] = None,
-                 instance_order: str = 'shuffle_once',
-                 adaptive_capping_slackfactor: float = 1.2,
-                 min_chall: int = 1,
-                 incumbent_selection: str = 'highest_executed_budget',
-                 identifier: int = 0,
-                 ) -> None:
+    def __init__(
+        self,
+        stats: Stats,
+        traj_logger: TrajLogger,
+        rng: np.random.RandomState,
+        instances: typing.List[str],
+        instance_specifics: typing.Mapping[str, str] = None,
+        cutoff: typing.Optional[float] = None,
+        deterministic: bool = False,
+        initial_budget: typing.Optional[float] = None,
+        max_budget: typing.Optional[float] = None,
+        eta: float = 3,
+        run_obj_time: bool = True,
+        n_seeds: typing.Optional[int] = None,
+        instance_order: str = "shuffle_once",
+        adaptive_capping_slackfactor: float = 1.2,
+        min_chall: int = 1,
+        incumbent_selection: str = "highest_executed_budget",
+        identifier: int = 0,
+        num_obj: int = 1,
+    ) -> None:
 
-        super().__init__(stats=stats,
-                         traj_logger=traj_logger,
-                         rng=rng,
-                         instances=instances,
-                         instance_specifics=instance_specifics,
-                         cutoff=cutoff,
-                         deterministic=deterministic,
-                         initial_budget=initial_budget,
-                         max_budget=max_budget,
-                         eta=eta,
-                         num_initial_challengers=None,  # initial challengers passed as None
-                         run_obj_time=run_obj_time,
-                         n_seeds=n_seeds,
-                         instance_order=instance_order,
-                         adaptive_capping_slackfactor=adaptive_capping_slackfactor,
-                         min_chall=min_chall,
-                         incumbent_selection=incumbent_selection,)
+        super().__init__(
+            stats=stats,
+            traj_logger=traj_logger,
+            rng=rng,
+            instances=instances,
+            instance_specifics=instance_specifics,
+            cutoff=cutoff,
+            deterministic=deterministic,
+            initial_budget=initial_budget,
+            max_budget=max_budget,
+            eta=eta,
+            num_initial_challengers=None,  # initial challengers passed as None
+            run_obj_time=run_obj_time,
+            n_seeds=n_seeds,
+            instance_order=instance_order,
+            adaptive_capping_slackfactor=adaptive_capping_slackfactor,
+            min_chall=min_chall,
+            incumbent_selection=incumbent_selection,
+            num_obj=num_obj,
+        )
 
         self.identifier = identifier
 
-        self.logger = logging.getLogger(
-            self.__module__ + "." + str(self.identifier) + "." + self.__class__.__name__)
+        self.logger = logging.getLogger(self.__module__ + "." + str(self.identifier) + "." + self.__class__.__name__)
 
         # to track completed hyperband iterations
         self.hb_iters = 0
         self.sh_intensifier = None  # type: _SuccessiveHalving # type: ignore[assignment]
 
-    def process_results(self,
-                        run_info: RunInfo,
-                        incumbent: typing.Optional[Configuration],
-                        run_history: RunHistory,
-                        time_bound: float,
-                        result: RunValue,
-                        log_traj: bool = True,
-                        ) -> \
-            typing.Tuple[Configuration, float]:
-        """
-        The intensifier stage will be updated based on the results/status
-        of a configuration execution.
-        Also, a incumbent will be determined.
+    def process_results(
+        self,
+        run_info: RunInfo,
+        incumbent: typing.Optional[Configuration],
+        run_history: RunHistory,
+        time_bound: float,
+        result: RunValue,
+        log_traj: bool = True,
+    ) -> typing.Tuple[Configuration, float]:
+        """The intensifier stage will be updated based on the results/status of a configuration
+        execution. Also, a incumbent will be determined.
 
         Parameters
         ----------
@@ -159,14 +166,15 @@ class _Hyperband(_SuccessiveHalving):
         inc_perf: float
             empirical performance of incumbent configuration
         """
-
         # run 1 iteration of successive halving
-        incumbent, inc_perf = self.sh_intensifier.process_results(run_info=run_info,
-                                                                  incumbent=incumbent,
-                                                                  run_history=run_history,
-                                                                  time_bound=time_bound,
-                                                                  result=result,
-                                                                  log_traj=log_traj)
+        incumbent, inc_perf = self.sh_intensifier.process_results(
+            run_info=run_info,
+            incumbent=incumbent,
+            run_history=run_history,
+            time_bound=time_bound,
+            result=result,
+            log_traj=log_traj,
+        )
         self.num_run += 1
 
         # reset if SH iteration is over, else update for next iteration
@@ -175,18 +183,19 @@ class _Hyperband(_SuccessiveHalving):
 
         return incumbent, inc_perf
 
-    def get_next_run(self,
-                     challengers: typing.Optional[typing.List[Configuration]],
-                     incumbent: Configuration,
-                     chooser: typing.Optional[EPMChooser],
-                     run_history: RunHistory,
-                     repeat_configs: bool = True,
-                     num_workers: int = 1,
-                     ) -> typing.Tuple[RunInfoIntent, RunInfo]:
-        """
-        Selects which challenger to use based on the iteration stage and set the iteration parameters.
-        First iteration will choose configurations from the ``chooser`` or input challengers,
-        while the later iterations pick top configurations from the previously selected challengers in that iteration
+    def get_next_run(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        incumbent: Configuration,
+        chooser: typing.Optional[EPMChooser],
+        run_history: RunHistory,
+        repeat_configs: bool = True,
+        num_workers: int = 1,
+    ) -> typing.Tuple[RunInfoIntent, RunInfo]:
+        """Selects which challenger to use based on the iteration stage and set the iteration
+        parameters. First iteration will choose configurations from the ``chooser`` or input
+        challengers, while the later iterations pick top configurations from the previously selected
+        challengers in that iteration.
 
         If no new run is available, the method returns a configuration of None.
 
@@ -213,14 +222,13 @@ class _Hyperband(_SuccessiveHalving):
         run_info: RunInfo
                An object that encapsulates necessary information for a config run
         """
-
         if num_workers > 1:
-            raise ValueError("HyperBand does not support more than 1 worker, yet "
-                             "the argument num_workers to get_next_run is {}".format(
-                                 num_workers
-                             ))
+            raise ValueError(
+                "HyperBand does not support more than 1 worker, yet "
+                "the argument num_workers to get_next_run is {}".format(num_workers)
+            )
 
-        if not hasattr(self, 's'):
+        if not hasattr(self, "s"):
             # initialize tracking variables
             self._update_stage()
 
@@ -232,7 +240,7 @@ class _Hyperband(_SuccessiveHalving):
             incumbent=incumbent,
             chooser=chooser,
             run_history=run_history,
-            repeat_configs=self.sh_intensifier.repeat_configs
+            repeat_configs=self.sh_intensifier.repeat_configs,
         )
 
         # For testing purposes, this attribute highlights whether a
@@ -243,17 +251,16 @@ class _Hyperband(_SuccessiveHalving):
         return intent, run_info
 
     def _update_stage(self, run_history: RunHistory = None) -> None:
-        """
-        Update tracking information for a new stage/iteration and update statistics.
-        This method is called to initialize stage variables and after all configurations
-        of a successive halving stage are completed.
+        """Update tracking information for a new stage/iteration and update statistics. This method
+        is called to initialize stage variables and after all configurations of a successive halving
+        stage are completed.
 
         Parameters
         ----------
          run_history : smac.runhistory.runhistory.RunHistory
             stores all runs we ran so far
         """
-        if not hasattr(self, 's'):
+        if not hasattr(self, "s"):
             # setting initial running budget for future iterations (s & s_max from Algorithm 1)
             self.s_max = int(np.floor(np.log(self.max_budget / self.initial_budget) / np.log(self.eta)))
             self.s = self.s_max
@@ -268,16 +275,18 @@ class _Hyperband(_SuccessiveHalving):
             self.s -= 1
 
         # compute min budget for new SH run
-        sh_initial_budget = self.eta ** -self.s * self.max_budget
+        sh_initial_budget = self.eta**-self.s * self.max_budget
         # sample challengers for next iteration (based on HpBandster package)
-        n_challengers = int(np.floor((self.s_max + 1) / (self.s + 1)) * self.eta ** self.s)
+        n_challengers = int(np.floor((self.s_max + 1) / (self.s + 1)) * self.eta**self.s)
 
         # Compute this for the next round
         n_configs_in_stage = n_challengers * np.power(self.eta, -np.linspace(0, self.s, self.s + 1))
         n_configs_in_stage = np.array(np.round(n_configs_in_stage), dtype=int).tolist()
 
-        self.logger.info('Hyperband iteration-step: %d-%d  with initial budget: %d' % (
-            self.hb_iters + 1, self.s_max - self.s + 1, sh_initial_budget))
+        self.logger.info(
+            "Hyperband iteration-step: %d-%d  with initial budget: %d"
+            % (self.hb_iters + 1, self.s_max - self.s + 1, sh_initial_budget)
+        )
 
         # creating a new Successive Halving intensifier with the current running budget
         self.sh_intensifier = _SuccessiveHalving(
@@ -291,7 +300,7 @@ class _Hyperband(_SuccessiveHalving):
             initial_budget=sh_initial_budget,
             max_budget=self.max_budget,
             eta=self.eta,
-            _all_budgets=self.all_budgets[(-self.s - 1):],
+            _all_budgets=self.all_budgets[(-self.s - 1) :],
             _n_configs_in_stage=n_configs_in_stage,
             num_initial_challengers=n_challengers,
             run_obj_time=self.run_obj_time,
@@ -304,7 +313,7 @@ class _Hyperband(_SuccessiveHalving):
 
 
 class Hyperband(ParallelScheduler):
-    """ Races multiple challengers against an incumbent using Hyperband method
+    """Races multiple challengers against an incumbent using Hyperband method.
 
     Implementation from "BOHB: Robust and Efficient Hyperparameter Optimization at Scale" (Falkner et al. 2018)
 
@@ -324,7 +333,7 @@ class Hyperband(ParallelScheduler):
     rng : np.random.RandomState
     instances : typing.List[str]
         list of all instance ids
-    instance_specifics : typing.Mapping[str,np.ndarray]
+    instance_specifics : typing.Mapping[str, str]
         mapping from instance name to instance specific string
     cutoff : typing.Optional[int]
         runtime cutoff of TA runs
@@ -357,38 +366,41 @@ class Hyperband(ParallelScheduler):
         * highest_budget - incumbent is selected only based on the highest budget
         * any_budget - incumbent is the best on any budget i.e., best performance regardless of budget
     """
-    def __init__(self,
-                 stats: Stats,
-                 traj_logger: TrajLogger,
-                 rng: np.random.RandomState,
-                 instances: typing.List[str],
-                 instance_specifics: typing.Mapping[str, np.ndarray] = None,
-                 cutoff: typing.Optional[float] = None,
-                 deterministic: bool = False,
-                 initial_budget: typing.Optional[float] = None,
-                 max_budget: typing.Optional[float] = None,
-                 eta: float = 3,
-                 run_obj_time: bool = True,
-                 n_seeds: typing.Optional[int] = None,
-                 instance_order: str = 'shuffle_once',
-                 adaptive_capping_slackfactor: float = 1.2,
-                 min_chall: int = 1,
-                 incumbent_selection: str = 'highest_executed_budget',
-                 ) -> None:
 
-        super().__init__(stats=stats,
-                         traj_logger=traj_logger,
-                         rng=rng,
-                         instances=instances,
-                         instance_specifics=instance_specifics,
-                         cutoff=cutoff,
-                         deterministic=deterministic,
-                         run_obj_time=run_obj_time,
-                         adaptive_capping_slackfactor=adaptive_capping_slackfactor,
-                         min_chall=min_chall)
+    def __init__(
+        self,
+        stats: Stats,
+        traj_logger: TrajLogger,
+        rng: np.random.RandomState,
+        instances: typing.List[str],
+        instance_specifics: typing.Mapping[str, str] = None,
+        cutoff: typing.Optional[float] = None,
+        deterministic: bool = False,
+        initial_budget: typing.Optional[float] = None,
+        max_budget: typing.Optional[float] = None,
+        eta: float = 3,
+        run_obj_time: bool = True,
+        n_seeds: typing.Optional[int] = None,
+        instance_order: str = "shuffle_once",
+        adaptive_capping_slackfactor: float = 1.2,
+        min_chall: int = 1,
+        incumbent_selection: str = "highest_executed_budget",
+    ) -> None:
 
-        self.logger = logging.getLogger(
-            self.__module__ + "." + self.__class__.__name__)
+        super().__init__(
+            stats=stats,
+            traj_logger=traj_logger,
+            rng=rng,
+            instances=instances,
+            instance_specifics=instance_specifics,
+            cutoff=cutoff,
+            deterministic=deterministic,
+            run_obj_time=run_obj_time,
+            adaptive_capping_slackfactor=adaptive_capping_slackfactor,
+            min_chall=min_chall,
+        )
+
+        self.logger = logging.getLogger(self.__module__ + "." + self.__class__.__name__)
 
         # Parameters for a new hyperband
         self.n_seeds = n_seeds
@@ -400,12 +412,9 @@ class Hyperband(ParallelScheduler):
         self.max_budget = max_budget
         self.eta = eta
 
-    def _get_intensifier_ranking(self, intensifier: AbstractRacer
-                                 ) -> typing.Tuple[int, int]:
-        """
-        Given a intensifier, returns how advance it is.
-        This metric will be used to determine what priority to
-        assign to the intensifier
+    def _get_intensifier_ranking(self, intensifier: AbstractRacer) -> typing.Tuple[int, int]:
+        """Given a intensifier, returns how advance it is. This metric will be used to determine
+        what priority to assign to the intensifier.
 
         Parameters
         ----------
@@ -429,26 +438,23 @@ class Hyperband(ParallelScheduler):
         # For hyperband, we use the internal successive halving as a criteria
         # to see how advanced this intensifier is
         stage = 0
-        if hasattr(intensifier.sh_intensifier, 'stage'):
+        if hasattr(intensifier.sh_intensifier, "stage"):
             # Newly created SuccessiveHalving objects have no stage
             stage = intensifier.sh_intensifier.stage
         return stage, len(intensifier.sh_intensifier.run_tracker)
 
     def _add_new_instance(self, num_workers: int) -> bool:
-        """
-        Decides if it is possible to add a new intensifier instance,
-        and adds it.
-        If a new intensifier instance is added, True is returned, else False.
+        """Decides if it is possible to add a new intensifier instance, and adds it. If a new
+        intensifier instance is added, True is returned, else False.
 
         Parameters
-        -----------
+        ----------
         num_workers: int
-            the maximum number of workers available
-            at a given time.
+            the maximum number of workers available at a given time.
 
         Returns
         -------
-            Whether or not a new instance was added
+            Whether or not a new instance was added.
         """
         if len(self.intensifier_instances) >= num_workers:
             return False
