@@ -1,27 +1,29 @@
-import logging
 import typing
+
+import logging
 from collections import Counter
 from enum import Enum
 
 import numpy as np
 
-from smac.stats.stats import Stats
-from smac.utils.constants import MAXINT
 from smac.configspace import Configuration
-from smac.runhistory.runhistory import (
-    InstSeedBudgetKey,
-    RunInfo,
-    RunHistory,
-    RunValue,
-    StatusType
-)
-from smac.utils.io.traj_logging import TrajLogger
 from smac.intensification.abstract_racer import (
     AbstractRacer,
     RunInfoIntent,
     _config_to_run_type,
 )
 from smac.optimizer.epm_configuration_chooser import EPMChooser
+from smac.runhistory.runhistory import (
+    InstSeedBudgetKey,
+    RunHistory,
+    RunInfo,
+    RunValue,
+    StatusType,
+)
+from smac.stats.stats import Stats
+from smac.utils.constants import MAXINT
+from smac.utils.io.traj_logging import TrajLogger
+from smac.utils.logging import format_array
 
 __author__ = "Katharina Eggensperger, Marius Lindauer"
 __copyright__ = "Copyright 2018, ML4AAD"
@@ -29,13 +31,14 @@ __license__ = "3-clause BSD"
 
 
 class NoMoreChallengers(Exception):
-    """Indicates that no more challengers are available
-    for the intensification to proceed"""
+    """Indicates that no more challengers are available for the intensification to proceed."""
+
     pass
 
 
 class IntensifierStage(Enum):
-    """Class to define different stages of intensifier"""
+    """Class to define different stages of intensifier."""
+
     RUN_FIRST_CONFIG = 0  # to replicate the old initial design
     RUN_INCUMBENT = 1  # Lines 3-7
     RUN_CHALLENGER = 2  # Lines 8-17
@@ -49,8 +52,7 @@ class IntensifierStage(Enum):
 
 
 class Intensifier(AbstractRacer):
-    """ Races challengers against an incumbent
-
+    r"""Races challengers against an incumbent.
     SMAC's intensification procedure, in detail:
 
     Procedure 2: Intensify(Θ_new, θ_inc, M, R, t_intensify, Π, cˆ)
@@ -79,86 +81,88 @@ class Intensifier(AbstractRacer):
     10        S_missing ← {instance, seed} pairs for which θ_inc was run before, but not θ_new
     11        S_torun ← random subset of S_missing of size min(N, size(S_missing))
     12        foreach (π, s) ∈ S_torun do R ← ExecuteRun(R, θ_new, π, s)
-    13        S_missing ← S_missing \ S_torun
+    13        S_missing ← S_missing \\ S_torun
     14        Π_common ← instances for which we previously ran both θ_inc and θ_new
     15        if cˆ(θ_new, Π_common) > cˆ(θ_inc, Π_common) then break
     16        else if S_missing = ∅ then θ_inc ← θ_new; break
     17        else N ← 2 · N
     18    if time spent in this call to this procedure exceeds t_intensify and i ≥ 2 then break
     19 return [R, θ_inc]
+
+    Parameters
+    ----------
+    stats: Stats
+        stats object
+    traj_logger: TrajLogger
+        TrajLogger object to log all new incumbents
+    rng : np.random.RandomState
+    instances : typing.List[str]
+        list of all instance ids
+    instance_specifics : typing.Mapping[str, str]
+        mapping from instance name to instance specific string
+    cutoff : int
+        runtime cutoff of TA runs
+    deterministic: bool
+        whether the TA is deterministic or not
+    run_obj_time: bool
+        whether the run objective is runtime or not (if true, apply adaptive capping)
+    always_race_against: Configuration
+        if incumbent changes race this configuration always against new incumbent;
+        can sometimes prevent over-tuning
+    use_ta_time_bound: bool,
+        if true, trust time reported by the target algorithms instead of
+        measuring the wallclock time for limiting the time of intensification
+    run_limit : int
+        Maximum number of target algorithm runs per call to intensify.
+    maxR : int
+        Maximum number of runs per config (summed over all calls to
+        intensifiy).
+    minR : int
+        Minimum number of run per config (summed over all calls to
+        intensify).
+    adaptive_capping_slackfactor: float
+        slack factor of adpative capping (factor * adpative cutoff)
+    min_chall: int
+        minimal number of challengers to be considered
+        (even if time_bound is exhausted earlier)
     """
 
-    def __init__(self,
-                 stats: Stats,
-                 traj_logger: TrajLogger,
-                 rng: np.random.RandomState,
-                 instances: typing.List[str],
-                 instance_specifics: typing.Mapping[str, np.ndarray] = None,
-                 cutoff: int = None,
-                 deterministic: bool = False,
-                 run_obj_time: bool = True,
-                 always_race_against: Configuration = None,
-                 run_limit: int = MAXINT,
-                 use_ta_time_bound: bool = False,
-                 minR: int = 1,
-                 maxR: int = 2000,
-                 adaptive_capping_slackfactor: float = 1.2,
-                 min_chall: int = 2,):
-        """ Creates an Intensifier object
+    def __init__(
+        self,
+        stats: Stats,
+        traj_logger: TrajLogger,
+        rng: np.random.RandomState,
+        instances: typing.List[str],
+        instance_specifics: typing.Mapping[str, str] = None,
+        cutoff: int = None,
+        deterministic: bool = False,
+        run_obj_time: bool = True,
+        always_race_against: Configuration = None,
+        run_limit: int = MAXINT,
+        use_ta_time_bound: bool = False,
+        minR: int = 1,
+        maxR: int = 2000,
+        adaptive_capping_slackfactor: float = 1.2,
+        min_chall: int = 2,
+        num_obj: int = 1,
+    ):
+        super().__init__(
+            stats=stats,
+            traj_logger=traj_logger,
+            rng=rng,
+            instances=instances,
+            instance_specifics=instance_specifics,
+            cutoff=cutoff,
+            deterministic=deterministic,
+            run_obj_time=run_obj_time,
+            minR=minR,
+            maxR=maxR,
+            adaptive_capping_slackfactor=adaptive_capping_slackfactor,
+            min_chall=min_chall,
+            num_obj=num_obj,
+        )
 
-        Parameters
-        ----------
-        stats: Stats
-            stats object
-        traj_logger: TrajLogger
-            TrajLogger object to log all new incumbents
-        rng : np.random.RandomState
-        instances : typing.List[str]
-            list of all instance ids
-        instance_specifics : typing.Mapping[str,np.ndarray]
-            mapping from instance name to instance specific string
-        cutoff : int
-            runtime cutoff of TA runs
-        deterministic: bool
-            whether the TA is deterministic or not
-        run_obj_time: bool
-            whether the run objective is runtime or not (if true, apply adaptive capping)
-        always_race_against: Configuration
-            if incumbent changes race this configuration always against new incumbent;
-            can sometimes prevent over-tuning
-        use_ta_time_bound: bool,
-            if true, trust time reported by the target algorithms instead of
-            measuring the wallclock time for limiting the time of intensification
-        run_limit : int
-            Maximum number of target algorithm runs per call to intensify.
-        maxR : int
-            Maximum number of runs per config (summed over all calls to
-            intensifiy).
-        minR : int
-            Minimum number of run per config (summed over all calls to
-            intensify).
-        adaptive_capping_slackfactor: float
-            slack factor of adpative capping (factor * adpative cutoff)
-        min_chall: int
-            minimal number of challengers to be considered
-            (even if time_bound is exhausted earlier)
-        """
-
-        super().__init__(stats=stats,
-                         traj_logger=traj_logger,
-                         rng=rng,
-                         instances=instances,
-                         instance_specifics=instance_specifics,
-                         cutoff=cutoff,
-                         deterministic=deterministic,
-                         run_obj_time=run_obj_time,
-                         minR=minR,
-                         maxR=maxR,
-                         adaptive_capping_slackfactor=adaptive_capping_slackfactor,
-                         min_chall=min_chall,)
-
-        self.logger = logging.getLogger(
-            self.__module__ + "." + self.__class__.__name__)
+        self.logger = logging.getLogger(self.__module__ + "." + self.__class__.__name__)
 
         # general attributes
         self.run_limit = run_limit
@@ -168,7 +172,7 @@ class Intensifier(AbstractRacer):
             raise ValueError("run_limit must be > 1")
 
         self.use_ta_time_bound = use_ta_time_bound
-        self.elapsed_time = 0.
+        self.elapsed_time = 0.0
 
         # stage variables
         # the intensification procedure is divided into 4 'stages':
@@ -192,18 +196,17 @@ class Intensifier(AbstractRacer):
         self.inc_sum_cost = np.inf
         self.N = -1
 
-    def get_next_run(self,
-                     challengers: typing.Optional[typing.List[Configuration]],
-                     incumbent: Configuration,
-                     chooser: typing.Optional[EPMChooser],
-                     run_history: RunHistory,
-                     repeat_configs: bool = True,
-                     num_workers: int = 1,
-                     ) -> typing.Tuple[RunInfoIntent, RunInfo]:
-        """
-        This procedure is in charge of generating a RunInfo object to comply
-        with lines 7 (in case stage is stage==RUN_INCUMBENT) or line 12
-        (In case of stage==RUN_CHALLENGER)
+    def get_next_run(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        incumbent: Configuration,
+        chooser: typing.Optional[EPMChooser],
+        run_history: RunHistory,
+        repeat_configs: bool = True,
+        num_workers: int = 1,
+    ) -> typing.Tuple[RunInfoIntent, RunInfo]:
+        """This procedure is in charge of generating a RunInfo object to comply with lines 7 (in
+        case stage is stage==RUN_INCUMBENT) or line 12 (In case of stage==RUN_CHALLENGER)
 
         A RunInfo object encapsulates the necessary information for a worker
         to execute the job, nevertheless, a challenger is not always available.
@@ -240,10 +243,10 @@ class Intensifier(AbstractRacer):
             An object that encapsulates necessary information for a config run
         """
         if num_workers > 1:
-            raise ValueError("Intensifier does not support more than 1 worker, yet "
-                             "the argument num_workers to get_next_run is {}".format(
-                                 num_workers
-                             ))
+            raise ValueError(
+                "Intensifier does not support more than 1 worker, yet "
+                "the argument num_workers to get_next_run is {}".format(num_workers)
+            )
 
         # If this function is called, it means the iteration is
         # not complete (we can be starting a new iteration, or re-running a
@@ -265,25 +268,16 @@ class Intensifier(AbstractRacer):
         # This new challenger is also assumed to be the incumbent
         if self.stage == IntensifierStage.RUN_FIRST_CONFIG:
             if incumbent is None:
-                self.logger.info(
-                    "First run, no incumbent provided;"
-                    " challenger is assumed to be the incumbent"
-                )
+                self.logger.info("First run, no incumbent provided;" " challenger is assumed to be the incumbent")
                 challenger, self.new_challenger = self.get_next_challenger(
                     challengers=challengers,
                     chooser=chooser,
                 )
                 incumbent = challenger
             else:
-                inc_runs = run_history.get_runs_for_config(
-                    incumbent,
-                    only_max_observed_budget=True
-                )
+                inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
                 if len(inc_runs) > 0:
-                    self.logger.debug(
-                        "Skipping RUN_FIRST_CONFIG stage since "
-                        "incumbent has already been ran"
-                    )
+                    self.logger.debug("Skipping RUN_FIRST_CONFIG stage since " "incumbent has already been ran")
                     self.stage = IntensifierStage.RUN_INCUMBENT
 
         # LINES 3-7
@@ -293,10 +287,7 @@ class Intensifier(AbstractRacer):
             # A modified version, that not only checks for maxR
             # but also makes sure that there are runnable instances,
             # that is, instances has not been exhausted
-            inc_runs = run_history.get_runs_for_config(
-                incumbent,
-                only_max_observed_budget=True
-            )
+            inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
 
             # Line 4
             available_insts = self._get_inc_available_inst(incumbent, run_history)
@@ -321,9 +312,7 @@ class Intensifier(AbstractRacer):
                 # This point marks the transitions from lines 3-7
                 # to 8-18.
 
-                self.logger.debug(
-                    "No further instance-seed pairs for incumbent available."
-                )
+                self.logger.debug("No further instance-seed pairs for incumbent available.")
 
                 self.stage = IntensifierStage.RUN_CHALLENGER
 
@@ -364,9 +353,7 @@ class Intensifier(AbstractRacer):
         # Skip the iteration if the challenger was previously run
         if challenger == incumbent and self.stage == IntensifierStage.RUN_CHALLENGER:
             self.challenger_same_as_incumbent = True
-            self.logger.debug(
-                "Challenger was the same as the current incumbent; Skipping challenger"
-            )
+            self.logger.debug("Challenger was the same as the current incumbent; Skipping challenger")
             return RunInfoIntent.SKIP, RunInfo(
                 config=None,
                 instance=None,
@@ -378,17 +365,14 @@ class Intensifier(AbstractRacer):
             )
 
         self.logger.debug("Intensify on %s", challenger)
-        if hasattr(challenger, 'origin'):
+        if hasattr(challenger, "origin"):
             self.logger.debug("Configuration origin: %s", challenger.origin)
 
         if self.stage in [IntensifierStage.RUN_CHALLENGER, IntensifierStage.RUN_BASIS]:
 
             if not self.to_run:
                 self.to_run, self.inc_sum_cost = self._get_instances_to_run(
-                    incumbent=incumbent,
-                    challenger=challenger,
-                    run_history=run_history,
-                    N=self.N
+                    incumbent=incumbent, challenger=challenger, run_history=run_history, N=self.N
                 )
 
             is_there_time_due_to_adaptive_cap = self._is_there_time_due_to_adaptive_cap(
@@ -408,10 +392,7 @@ class Intensifier(AbstractRacer):
                     # we discard all the forthcoming runs.
                     self.to_run = []
                     self.stage = IntensifierStage.RUN_INCUMBENT
-                    self.logger.debug(
-                        "Stop challenger itensification due "
-                        "to adaptive capping."
-                    )
+                    self.logger.debug("Stop challenger itensification due " "to adaptive capping.")
 
                 # Nevertheless, if there are no more instances to run,
                 # we might need to comply with line 17 and keep running the
@@ -465,20 +446,19 @@ class Intensifier(AbstractRacer):
                     budget=0.0,
                 )
         else:
-            raise ValueError('No valid stage found!')
+            raise ValueError("No valid stage found!")
 
-    def process_results(self,
-                        run_info: RunInfo,
-                        incumbent: typing.Optional[Configuration],
-                        run_history: RunHistory,
-                        time_bound: float,
-                        result: RunValue,
-                        log_traj: bool = True,
-                        ) -> \
-            typing.Tuple[Configuration, float]:
-        """
-        The intensifier stage will be updated based on the results/status
-        of a configuration execution.
+    def process_results(
+        self,
+        run_info: RunInfo,
+        incumbent: typing.Optional[Configuration],
+        run_history: RunHistory,
+        time_bound: float,
+        result: RunValue,
+        log_traj: bool = True,
+    ) -> typing.Tuple[Configuration, float]:
+        """The intensifier stage will be updated based on the results/status of a configuration
+        execution.
 
         During intensification, the following can happen:
 
@@ -518,14 +498,13 @@ class Intensifier(AbstractRacer):
         """
         if self.stage == IntensifierStage.PROCESS_FIRST_CONFIG_RUN:
             if incumbent is None:
-                self.logger.info(
-                    "First run, no incumbent provided;"
-                    " challenger is assumed to be the incumbent"
-                )
+                self.logger.info("First run, no incumbent provided;" " challenger is assumed to be the incumbent")
                 incumbent = run_info.config
 
-        if self.stage in [IntensifierStage.PROCESS_INCUMBENT_RUN,
-                          IntensifierStage.PROCESS_FIRST_CONFIG_RUN]:
+        if self.stage in [
+            IntensifierStage.PROCESS_INCUMBENT_RUN,
+            IntensifierStage.PROCESS_FIRST_CONFIG_RUN,
+        ]:
             self._ta_time += result.time
             self.num_run += 1
             self._process_inc_run(
@@ -539,10 +518,7 @@ class Intensifier(AbstractRacer):
             self.num_chall_run += 1
             if result.status == StatusType.CAPPED:
                 # move on to the next iteration
-                self.logger.debug(
-                    "Challenger itensification timed out due "
-                    "to adaptive capping."
-                )
+                self.logger.debug("Challenger itensification timed out due " "to adaptive capping.")
                 self.stage = IntensifierStage.RUN_INCUMBENT
             else:
 
@@ -554,7 +530,7 @@ class Intensifier(AbstractRacer):
                     log_traj=log_traj,
                 )
 
-        self.elapsed_time += (result.endtime - result.starttime)
+        self.elapsed_time += result.endtime - result.starttime
         # check if 1 intensification run is complete - line 18
         # this is different to regular SMAC as it requires at least successful challenger run,
         # which is necessary to work on a fixed grid of configurations.
@@ -569,10 +545,9 @@ class Intensifier(AbstractRacer):
 
             if not self.use_ta_time_bound and self.elapsed_time - time_bound >= 0:
                 self.logger.debug(
-                    "Wallclock time limit for intensification reached "
-                    "(used: %f sec, available: %f sec)",
+                    "Wallclock time limit for intensification reached " "(used: %f sec, available: %f sec)",
                     self.elapsed_time,
-                    time_bound
+                    time_bound,
                 )
 
                 self._next_iteration()
@@ -580,7 +555,8 @@ class Intensifier(AbstractRacer):
             elif self._ta_time - time_bound >= 0:
                 self.logger.debug(
                     "TA time limit for intensification reached (used: %f sec, available: %f sec)",
-                    self._ta_time, time_bound
+                    self._ta_time,
+                    time_bound,
                 )
 
                 self._next_iteration()
@@ -589,11 +565,11 @@ class Intensifier(AbstractRacer):
 
         return incumbent, inc_perf
 
-    def _get_next_inc_run(self,
-                          available_insts: typing.List[str],
-                          ) -> typing.Tuple[str, int, typing.Optional[float]]:
-        """Method to extract the next seed/instance in which a
-        incumbent run most be evaluated.
+    def _get_next_inc_run(
+        self,
+        available_insts: typing.List[str],
+    ) -> typing.Tuple[str, int, typing.Optional[float]]:
+        """Method to extract the next seed/instance in which a incumbent run most be evaluated.
 
         Parameters
         ----------
@@ -608,7 +584,6 @@ class Intensifier(AbstractRacer):
             Seed in which to evaluate the instance
         cutoff: Optional[float]
             Max time for a given instance/seed pair
-
         """
         # Line 5 - and avoid https://github.com/numpy/numpy/issues/10791
         _idx = self.rs.choice(len(available_insts))
@@ -621,11 +596,7 @@ class Intensifier(AbstractRacer):
             next_seed = int(self.rs.randint(low=0, high=MAXINT, size=1)[0])
 
         # Line 7
-        self.logger.debug(
-            "Add run of incumbent for instance={}".format(
-                next_instance
-            )
-        )
+        self.logger.debug("Add run of incumbent for instance={}".format(next_instance))
         if self.stage == IntensifierStage.RUN_FIRST_CONFIG:
             self.stage = IntensifierStage.PROCESS_FIRST_CONFIG_RUN
         else:
@@ -639,8 +610,7 @@ class Intensifier(AbstractRacer):
         run_history: RunHistory,
         log_traj: bool = True,
     ) -> typing.List[str]:
-        """
-        Implementation of line 4 of Intensification
+        """Implementation of line 4 of Intensification.
 
         This method queries the inc runs in the run history
         and return the pending instances if any is available
@@ -653,16 +623,13 @@ class Intensifier(AbstractRacer):
             stores all runs we ran so far
         log_traj: bool
             Whether to log changes of incumbents in trajectory
-
         """
         # Line 4
         # find all instances that have the most runs on the inc
-        inc_runs = run_history.get_runs_for_config(
-            incumbent,
-            only_max_observed_budget=True
-        )
+        inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
         inc_inst = [s.instance for s in inc_runs]
         inc_inst = list(Counter(inc_inst).items())
+
         inc_inst.sort(key=lambda x: x[1], reverse=True)
         try:
             max_runs = inc_inst[0][1]
@@ -679,13 +646,13 @@ class Intensifier(AbstractRacer):
             available_insts = self.instances
         return available_insts
 
-    def _process_inc_run(self,
-                         incumbent: Configuration,
-                         run_history: RunHistory,
-                         log_traj: bool = True,
-                         ) -> None:
-        """Method to process the results of a challenger that races
-        an incumbent
+    def _process_inc_run(
+        self,
+        incumbent: Configuration,
+        run_history: RunHistory,
+        log_traj: bool = True,
+    ) -> None:
+        """Method to process the results of a challenger that races an incumbent.
 
         Parameters
         ----------
@@ -695,17 +662,18 @@ class Intensifier(AbstractRacer):
             stores all runs we ran so far
         log_traj: bool
             Whether to log changes of incumbents in trajectory
-
         """
         # output estimated performance of incumbent
         inc_runs = run_history.get_runs_for_config(incumbent, only_max_observed_budget=True)
         inc_perf = run_history.get_cost(incumbent)
-        self.logger.info("Updated estimated cost of incumbent on %d runs: %.4f"
-                         % (len(inc_runs), inc_perf))
+        format_value = format_array(inc_perf)
+        self.logger.info(f"Updated estimated cost of incumbent on {len(inc_runs)} runs: {format_value}")
 
         # if running first configuration, go to next stage after 1st run
-        if self.stage in [IntensifierStage.RUN_FIRST_CONFIG,
-                          IntensifierStage.PROCESS_FIRST_CONFIG_RUN]:
+        if self.stage in [
+            IntensifierStage.RUN_FIRST_CONFIG,
+            IntensifierStage.PROCESS_FIRST_CONFIG_RUN,
+        ]:
             self.stage = IntensifierStage.RUN_INCUMBENT
             self._next_iteration()
         else:
@@ -716,12 +684,7 @@ class Intensifier(AbstractRacer):
             else:
                 self.stage = IntensifierStage.RUN_INCUMBENT
 
-        self._compare_configs(
-            incumbent=incumbent,
-            challenger=incumbent,
-            run_history=run_history,
-            log_traj=log_traj
-        )
+        self._compare_configs(incumbent=incumbent, challenger=incumbent, run_history=run_history, log_traj=log_traj)
 
     def _get_next_racer(
         self,
@@ -730,8 +693,8 @@ class Intensifier(AbstractRacer):
         run_history: RunHistory,
         log_traj: bool = True,
     ) -> typing.Tuple[Configuration, str, int, typing.Optional[float]]:
-        """Method to return the next config setting to
-        aggressively race challenger against incumbent.
+        """Method to return the next config setting to aggressively race challenger against
+        incumbent.
 
         Parameters
         ----------
@@ -755,17 +718,12 @@ class Intensifier(AbstractRacer):
         cutoff: Optional[float]
             Max time for a given instance/seed pair
         """
-
         # By the time this function is called, the run history might
         # have shifted. Re-populate the list if necessary
         if not self.to_run:
-
             # Lines 10/11
             self.to_run, self.inc_sum_cost = self._get_instances_to_run(
-                incumbent=incumbent,
-                challenger=challenger,
-                run_history=run_history,
-                N=self.N
+                incumbent=incumbent, challenger=challenger, run_history=run_history, N=self.N
             )
 
         # Run challenger on all <instance, seed> to run
@@ -773,11 +731,9 @@ class Intensifier(AbstractRacer):
 
         cutoff = self.cutoff
         if self.run_obj_time:
-            cutoff = self._adapt_cutoff(challenger=challenger,
-                                        run_history=run_history,
-                                        inc_sum_cost=self.inc_sum_cost)
+            cutoff = self._adapt_cutoff(challenger=challenger, run_history=run_history, inc_sum_cost=self.inc_sum_cost)
 
-        self.logger.debug('Cutoff for challenger: %s' % str(cutoff))
+        self.logger.debug("Cutoff for challenger: %s" % str(cutoff))
 
         self.logger.debug("Add run of challenger")
 
@@ -789,10 +745,8 @@ class Intensifier(AbstractRacer):
         challenger: Configuration,
         run_history: RunHistory,
     ) -> bool:
-        """ A check to see if there is no more time for a challenger
-        given the fact, that we are optimizing time and the
-        incumbent looks more promising
-        Line 18
+        """A check to see if there is no more time for a challenger given the fact, that we are
+        optimizing time and the incumbent looks more promising Line 18.
 
         Parameters
         ----------
@@ -809,24 +763,21 @@ class Intensifier(AbstractRacer):
         if not self.run_obj_time:
             return True
 
-        cutoff = self._adapt_cutoff(
-            challenger=challenger,
-            run_history=run_history,
-            inc_sum_cost=self.inc_sum_cost
-        )
+        cutoff = self._adapt_cutoff(challenger=challenger, run_history=run_history, inc_sum_cost=self.inc_sum_cost)
         if cutoff is not None and cutoff <= 0:
             return False
         else:
             return True
 
-    def _process_racer_results(self,
-                               challenger: Configuration,
-                               incumbent: Configuration,
-                               run_history: RunHistory,
-                               log_traj: bool = True,
-                               ) -> typing.Optional[Configuration]:
-        """ Process the result of a racing configuration against the
-        current incumbent. Might propose a new incumbent.
+    def _process_racer_results(
+        self,
+        challenger: Configuration,
+        incumbent: Configuration,
+        run_history: RunHistory,
+        log_traj: bool = True,
+    ) -> typing.Optional[Configuration]:
+        """Process the result of a racing configuration against the current incumbent. Might propose
+        a new incumbent.
 
         Parameters
         ----------
@@ -847,52 +798,67 @@ class Intensifier(AbstractRacer):
         # if all <instance, seed> have been run, compare challenger performance
         if not self.to_run:
             new_incumbent = self._compare_configs(
-                incumbent=incumbent, challenger=challenger,
+                incumbent=incumbent,
+                challenger=challenger,
                 run_history=run_history,
-                log_traj=log_traj)
+                log_traj=log_traj,
+            )
 
             # update intensification stage
             if new_incumbent == incumbent:
                 # move on to the next iteration
                 self.stage = IntensifierStage.RUN_INCUMBENT
                 self.continue_challenger = False
-                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, but worse than incumbent',
-                                  len(chal_runs), chal_perf)
+                self.logger.debug(
+                    "Estimated cost of challenger on %d runs: %.4f, but worse than incumbent",
+                    len(chal_runs),
+                    chal_perf,
+                )
 
             elif new_incumbent == challenger:
                 # New incumbent found
                 incumbent = challenger
                 self.continue_challenger = False
                 # compare against basis configuration if provided, else go to next iteration
-                if self.always_race_against and \
-                        self.always_race_against != challenger:
+                if self.always_race_against and self.always_race_against != challenger:
                     self.stage = IntensifierStage.RUN_BASIS
                 else:
                     self.stage = IntensifierStage.RUN_INCUMBENT
-                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, becomes new incumbent',
-                                  len(chal_runs), chal_perf)
+                self.logger.debug(
+                    "Estimated cost of challenger on %d runs: %.4f, becomes new incumbent",
+                    len(chal_runs),
+                    chal_perf,
+                )
 
             else:  # Line 17
                 # challenger is not worse, continue
                 self.N = 2 * self.N
                 self.continue_challenger = True
-                self.logger.debug('Estimated cost of challenger on %d runs: %.4f, adding %d runs to the queue',
-                                  len(chal_runs), chal_perf, self.N / 2)
+                self.logger.debug(
+                    "Estimated cost of challenger on %d runs: %.4f, adding %d runs to the queue",
+                    len(chal_runs),
+                    chal_perf,
+                    self.N / 2,
+                )
         else:
-            self.logger.debug('Estimated cost of challenger on %d runs: %.4f, still %d runs to go (continue racing)',
-                              len(chal_runs), chal_perf, len(self.to_run))
+            self.logger.debug(
+                "Estimated cost of challenger on %d runs: %.4f, still %d runs to go (continue racing)",
+                len(chal_runs),
+                chal_perf,
+                len(self.to_run),
+            )
 
         return incumbent
 
-    def _get_instances_to_run(self,
-                              challenger: Configuration,
-                              incumbent: Configuration,
-                              N: int,
-                              run_history: RunHistory,
-                              ) -> typing.Tuple[typing.List[InstSeedBudgetKey], float]:
-        """
-        Returns the minimum list of <instance, seed> pairs to run the challenger on
-        before comparing it with the incumbent
+    def _get_instances_to_run(
+        self,
+        challenger: Configuration,
+        incumbent: Configuration,
+        N: int,
+        run_history: RunHistory,
+    ) -> typing.Tuple[typing.List[InstSeedBudgetKey], float]:
+        """Returns the minimum list of <instance, seed> pairs to run the challenger on before
+        comparing it with the incumbent.
 
         Parameters
         ----------
@@ -922,9 +888,9 @@ class Intensifier(AbstractRacer):
         # Line 11
         self.rs.shuffle(missing_runs)
         if N < 0:
-            raise ValueError('Argument N must not be smaller than zero, but is %s' % str(N))
-        to_run = missing_runs[:min(N, len(missing_runs))]
-        missing_runs = missing_runs[min(N, len(missing_runs)):]
+            raise ValueError("Argument N must not be smaller than zero, but is %s" % str(N))
+        to_run = missing_runs[: min(N, len(missing_runs))]
+        missing_runs = missing_runs[min(N, len(missing_runs)) :]
 
         # for adaptive capping
         # because of efficiency computed here
@@ -937,14 +903,12 @@ class Intensifier(AbstractRacer):
 
         return to_run, inc_sum_cost
 
-    def get_next_challenger(self,
-                            challengers: typing.Optional[typing.List[Configuration]],
-                            chooser: typing.Optional[EPMChooser],
-                            ) -> \
-            typing.Tuple[typing.Optional[Configuration], bool]:
-        """
-        This function returns the next challenger, that should be exercised
-        though lines 8-17.
+    def get_next_challenger(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        chooser: typing.Optional[EPMChooser],
+    ) -> typing.Tuple[typing.Optional[Configuration], bool]:
+        """This function returns the next challenger, that should be exercised though lines 8-17.
 
         It does so by populating configs_to_run, which is a pool of configuration
         from which the racer will sample. Each configuration within configs_to_run,
@@ -972,13 +936,10 @@ class Intensifier(AbstractRacer):
             next configuration to evaluate
         bool
             flag telling if the configuration is newly sampled or one currently being tracked
-
         """
-
         # select new configuration when entering 'race challenger' stage
         # or for the first run
-        if not self.current_challenger or \
-                (self.stage == IntensifierStage.RUN_CHALLENGER and not self.to_run):
+        if not self.current_challenger or (self.stage == IntensifierStage.RUN_CHALLENGER and not self.to_run):
 
             # this is a new intensification run, get the next list of configurations to run
             if self.update_configs_to_run:
@@ -1006,13 +967,13 @@ class Intensifier(AbstractRacer):
         # return currently running challenger
         return self.current_challenger, False
 
-    def _generate_challengers(self,
-                              challengers: typing.Optional[typing.List[Configuration]],
-                              chooser: typing.Optional[EPMChooser]) \
-            -> _config_to_run_type:
-        """
-        Retuns a sequence of challengers to use in intensification
-        If challengers are not provided, then optimizer will be used to generate the challenger list
+    def _generate_challengers(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        chooser: typing.Optional[EPMChooser],
+    ) -> _config_to_run_type:
+        """Retuns a sequence of challengers to use in intensification If challengers are not
+        provided, then optimizer will be used to generate the challenger list.
 
         Parameters
         ----------
@@ -1026,7 +987,6 @@ class Intensifier(AbstractRacer):
         typing.Optional[typing.Generator[Configuration]]
             A generator containing the next challengers to use
         """
-
         if challengers:
             # iterate over challengers provided
             self.logger.debug("Using challengers provided")
@@ -1036,14 +996,12 @@ class Intensifier(AbstractRacer):
             self.logger.debug("Generating new challenger from optimizer")
             chall_gen = chooser.choose_next()
         else:
-            raise ValueError('No configurations/chooser provided. Cannot generate challenger!')
+            raise ValueError("No configurations/chooser provided. Cannot generate challenger!")
 
         return chall_gen
 
     def _next_iteration(self) -> None:
-        """
-        Updates tracking variables at the end of an intensification run
-        """
+        """Updates tracking variables at the end of an intensification run."""
         # track iterations
         self.n_iters += 1
         self.iteration_done = True
@@ -1057,5 +1015,4 @@ class Intensifier(AbstractRacer):
         self.elapsed_time = 0
         self._ta_time = 0.0
 
-        self.stats.update_average_configs_per_intensify(
-            n_configs=self._chall_indx)
+        self.stats.update_average_configs_per_intensify(n_configs=self._chall_indx)
