@@ -1,13 +1,14 @@
 import typing
+
 import warnings
 
 import numpy as np
 
+from smac.configspace import Configuration
 from smac.intensification.abstract_racer import AbstractRacer, RunInfoIntent
 from smac.optimizer.epm_configuration_chooser import EPMChooser
-from smac.stats.stats import Stats
-from smac.configspace import Configuration
 from smac.runhistory.runhistory import RunHistory, RunInfo, RunValue
+from smac.stats.stats import Stats
 from smac.utils.io.traj_logging import TrajLogger
 
 __copyright__ = "Copyright 2021, AutoML.org Freiburg-Hannover"
@@ -15,9 +16,7 @@ __license__ = "3-clause BSD"
 
 
 class ParallelScheduler(AbstractRacer):
-
-    """Common Racer class for Intensifiers that will schedule configurations on
-    a parallel fashion.
+    """Common Racer class for Intensifiers that will schedule configurations on a parallel fashion.
 
     This class instantiates intensifier objects on a need basis, that is, to
     prevent workers from being idle. This intensifier objects will give configurations
@@ -32,7 +31,7 @@ class ParallelScheduler(AbstractRacer):
     rng : np.random.RandomState
     instances : typing.List[str]
         list of all instance ids
-    instance_specifics : typing.Mapping[str,np.ndarray]
+    instance_specifics : typing.Mapping[str, str]
         mapping from instance name to instance specific string
     cutoff : typing.Optional[int]
         cutoff of TA runs
@@ -70,52 +69,58 @@ class ParallelScheduler(AbstractRacer):
         * any_budget - incumbent is the best on any budget i.e., best performance regardless of budget
     """
 
-    def __init__(self,
-                 stats: Stats,
-                 traj_logger: TrajLogger,
-                 rng: np.random.RandomState,
-                 instances: typing.List[str],
-                 instance_specifics: typing.Mapping[str, np.ndarray] = None,
-                 cutoff: typing.Optional[float] = None,
-                 deterministic: bool = False,
-                 initial_budget: typing.Optional[float] = None,
-                 max_budget: typing.Optional[float] = None,
-                 eta: float = 3,
-                 num_initial_challengers: typing.Optional[int] = None,
-                 run_obj_time: bool = True,
-                 n_seeds: typing.Optional[int] = None,
-                 instance_order: typing.Optional[str] = 'shuffle_once',
-                 adaptive_capping_slackfactor: float = 1.2,
-                 inst_seed_pairs: typing.Optional[typing.List[typing.Tuple[str, int]]] = None,
-                 min_chall: int = 1,
-                 incumbent_selection: str = 'highest_executed_budget',
-                 ) -> None:
+    def __init__(
+        self,
+        stats: Stats,
+        traj_logger: TrajLogger,
+        rng: np.random.RandomState,
+        instances: typing.List[str],
+        instance_specifics: typing.Mapping[str, str] = None,
+        cutoff: typing.Optional[float] = None,
+        deterministic: bool = False,
+        initial_budget: typing.Optional[float] = None,
+        max_budget: typing.Optional[float] = None,
+        eta: float = 3,
+        num_initial_challengers: typing.Optional[int] = None,
+        run_obj_time: bool = True,
+        n_seeds: typing.Optional[int] = None,
+        instance_order: typing.Optional[str] = "shuffle_once",
+        adaptive_capping_slackfactor: float = 1.2,
+        inst_seed_pairs: typing.Optional[typing.List[typing.Tuple[str, int]]] = None,
+        min_chall: int = 1,
+        incumbent_selection: str = "highest_executed_budget",
+        num_obj: int = 1,
+    ) -> None:
 
-        super().__init__(stats=stats,
-                         traj_logger=traj_logger,
-                         rng=rng,
-                         instances=instances,
-                         instance_specifics=instance_specifics,
-                         cutoff=cutoff,
-                         deterministic=deterministic,
-                         run_obj_time=run_obj_time,
-                         adaptive_capping_slackfactor=adaptive_capping_slackfactor,
-                         min_chall=min_chall)
+        super().__init__(
+            stats=stats,
+            traj_logger=traj_logger,
+            rng=rng,
+            instances=instances,
+            instance_specifics=instance_specifics,
+            cutoff=cutoff,
+            deterministic=deterministic,
+            run_obj_time=run_obj_time,
+            adaptive_capping_slackfactor=adaptive_capping_slackfactor,
+            min_chall=min_chall,
+            num_obj=num_obj,
+        )
 
         # We have a pool of instances that yield configurations ot run
         self.intensifier_instances = {}  # type: typing.Dict[int, AbstractRacer]
+        self.print_worker_warning = True
 
-    def get_next_run(self,
-                     challengers: typing.Optional[typing.List[Configuration]],
-                     incumbent: Configuration,
-                     chooser: typing.Optional[EPMChooser],
-                     run_history: RunHistory,
-                     repeat_configs: bool = False,
-                     num_workers: int = 1,
-                     ) -> typing.Tuple[RunInfoIntent, RunInfo]:
-        """
-        This procedure decides from which instance to pick a config,
-        in order to determine the next run.
+    def get_next_run(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        incumbent: Configuration,
+        chooser: typing.Optional[EPMChooser],
+        run_history: RunHistory,
+        repeat_configs: bool = False,
+        num_workers: int = 1,
+    ) -> typing.Tuple[RunInfoIntent, RunInfo]:
+        """This procedure decides from which instance to pick a config, in order to determine the
+        next run.
 
         To prevent having idle workers, this procedure creates new instances
         up to the maximum number of workers available.
@@ -147,21 +152,19 @@ class ParallelScheduler(AbstractRacer):
             An object that encapsulates the minimum information to
             evaluate a configuration
         """
-
-        if num_workers <= 1:
-            warnings.warn("{} is intended to be used "
-                          "with more than 1 worker but num_workers={}".format(
-                              self.__class__.__name__,
-                              num_workers
-                          ))
+        if num_workers <= 1 and self.print_worker_warning:
+            warnings.warn(
+                f"{self.__class__.__name__} is executed with {num_workers} workers only. "
+                "Consider to use pynisher to use all available workers."
+            )
+            self.print_worker_warning = False
 
         # If repeat_configs is True, that means that not only self can repeat
         # configurations, but also in the context of multiprocessing, N
         # intensifier instances will also share configurations. The later
         # is not supported
         if repeat_configs:
-            raise ValueError(
-                "repeat_configs==True is not supported for parallel execution")
+            raise ValueError("repeat_configs==True is not supported for parallel execution")
 
         # First get a config to run from a SH instance
         for i in self._sort_instances_by_stage(self.intensifier_instances):
@@ -204,18 +207,17 @@ class ParallelScheduler(AbstractRacer):
             budget=0.0,
         )
 
-    def process_results(self,
-                        run_info: RunInfo,
-                        incumbent: typing.Optional[Configuration],
-                        run_history: RunHistory,
-                        time_bound: float,
-                        result: RunValue,
-                        log_traj: bool = True,
-                        ) -> \
-            typing.Tuple[Configuration, float]:
-        """
-        The intensifier stage will be updated based on the results/status
-        of a configuration execution.
+    def process_results(
+        self,
+        run_info: RunInfo,
+        incumbent: typing.Optional[Configuration],
+        run_history: RunHistory,
+        time_bound: float,
+        result: RunValue,
+        log_traj: bool = True,
+    ) -> typing.Tuple[Configuration, float]:
+        """The intensifier stage will be updated based on the results/status of a configuration
+        execution.
 
         To do so, this procedures redirects the result argument, to the
         respective intensifier object that generated the original config.
@@ -249,7 +251,6 @@ class ParallelScheduler(AbstractRacer):
         inc_perf: float
             empirical performance of incumbent configuration
         """
-
         return self.intensifier_instances[run_info.source_id].process_results(
             run_info=run_info,
             incumbent=incumbent,
@@ -260,10 +261,8 @@ class ParallelScheduler(AbstractRacer):
         )
 
     def _add_new_instance(self, num_workers: int) -> bool:
-        """
-        Decides if it is possible to add a new intensifier instance,
-        and adds it.
-        If a new intensifier instance is added, True is returned, else False.
+        """Decides if it is possible to add a new intensifier instance, and adds it. If a new
+        intensifier instance is added, True is returned, else False.
 
         Parameters
         ----------
@@ -277,12 +276,9 @@ class ParallelScheduler(AbstractRacer):
         """
         raise NotImplementedError()
 
-    def _get_intensifier_ranking(self, intensifier: AbstractRacer
-                                 ) -> typing.Tuple[int, int]:
-        """
-        Given a intensifier, returns how advance it is.
-        This metric will be used to determine what priority to
-        assign to the intensifier
+    def _get_intensifier_ranking(self, intensifier: AbstractRacer) -> typing.Tuple[int, int]:
+        """Given a intensifier, returns how advance it is. This metric will be used to determine
+        what priority to assign to the intensifier.
 
         Parameters
         ----------
@@ -302,13 +298,10 @@ class ParallelScheduler(AbstractRacer):
         """
         raise NotImplementedError()
 
-    def _sort_instances_by_stage(self, instances: typing.Dict[int, AbstractRacer]
-                                 ) -> typing.List[int]:
-        """
-        This procedure dictates what SH to prioritize in
-        launching jobs. It prioritizes resource allocation to
-        SH instances that have higher stages. In case of tie,
-        we prioritize the SH instance with more launched configs
+    def _sort_instances_by_stage(self, instances: typing.Dict[int, AbstractRacer]) -> typing.List[int]:
+        """This procedure dictates what SH to prioritize in launching jobs. It prioritizes resource
+        allocation to SH instances that have higher stages. In case of tie, we prioritize the SH
+        instance with more launched configs.
 
         Parameters
         ----------
@@ -319,9 +312,7 @@ class ParallelScheduler(AbstractRacer):
         -------
         List:
             The order in which to query for new jobs
-
         """
-
         # This function might be called when no intensifier instances
         # exist (first iteration), so we return an empty list in that case
         if len(instances) == 0:
