@@ -15,9 +15,7 @@ __license__ = "3-clause BSD"
 
 
 class SimpleIntensifier(AbstractRacer):
-
-    """ Performs the traditional Bayesian Optimization loop, without
-        instance/seed intensification
+    """Performs the traditional Bayesian Optimization loop, without instance/seed intensification.
 
     Parameters
     ----------
@@ -28,7 +26,7 @@ class SimpleIntensifier(AbstractRacer):
     rng : np.random.RandomState
     instances : typing.List[str]
         list of all instance ids
-    instance_specifics : typing.Mapping[str,np.ndarray]
+    instance_specifics : typing.Mapping[str, str]
         mapping from instance name to instance specific string
     cutoff : typing.Optional[int]
         cutoff of TA runs
@@ -38,29 +36,34 @@ class SimpleIntensifier(AbstractRacer):
         whether the run objective is runtime or not (if true, apply adaptive capping)
     """
 
-    def __init__(self,
-                 stats: Stats,
-                 traj_logger: TrajLogger,
-                 rng: np.random.RandomState,
-                 instances: typing.List[str],
-                 instance_specifics: typing.Mapping[str, np.ndarray] = None,
-                 cutoff: typing.Optional[float] = None,
-                 deterministic: bool = False,
-                 run_obj_time: bool = True,
-                 **kwargs: typing.Any
-                 ) -> None:
+    def __init__(
+        self,
+        stats: Stats,
+        traj_logger: TrajLogger,
+        rng: np.random.RandomState,
+        instances: typing.List[str],
+        instance_specifics: typing.Mapping[str, str] = None,
+        cutoff: typing.Optional[float] = None,
+        deterministic: bool = False,
+        run_obj_time: bool = True,
+        num_obj: int = 1,
+        **kwargs: typing.Any,
+    ) -> None:
 
-        super().__init__(stats=stats,
-                         traj_logger=traj_logger,
-                         rng=rng,
-                         instances=instances,
-                         instance_specifics=instance_specifics,
-                         cutoff=cutoff,
-                         deterministic=deterministic,
-                         run_obj_time=run_obj_time,
-                         adaptive_capping_slackfactor=1.0,
-                         min_chall=1,
-                         )
+        super().__init__(
+            stats=stats,
+            traj_logger=traj_logger,
+            rng=rng,
+            instances=instances,
+            instance_specifics=instance_specifics,
+            cutoff=cutoff,
+            deterministic=deterministic,
+            run_obj_time=run_obj_time,
+            adaptive_capping_slackfactor=1.0,
+            min_chall=1,
+            num_obj=num_obj,
+        )
+        # Simple intensifier does not require comparing run results, thus we could simply ignore num_obj here
 
         # We want to control the number of runs that are sent to
         # the workers. At any time, we want to make sure that if there
@@ -68,19 +71,17 @@ class SimpleIntensifier(AbstractRacer):
         # Below variable tracks active runs not processed
         self.run_tracker = {}  # type: typing.Dict[typing.Tuple[Configuration, str, int, float], bool]
 
-    def process_results(self,
-                        run_info: RunInfo,
-                        incumbent: typing.Optional[Configuration],
-                        run_history: RunHistory,
-                        time_bound: float,
-                        result: RunValue,
-                        log_traj: bool = True,
-                        ) -> \
-            typing.Tuple[Configuration, float]:
-        """
-        The intensifier stage will be updated based on the results/status
-        of a configuration execution.
-        Also, a incumbent will be determined.
+    def process_results(
+        self,
+        run_info: RunInfo,
+        incumbent: typing.Optional[Configuration],
+        run_history: RunHistory,
+        time_bound: float,
+        result: RunValue,
+        log_traj: bool = True,
+    ) -> typing.Tuple[Configuration, float]:
+        """The intensifier stage will be updated based on the results/status of a configuration
+        execution. Also, a incumbent will be determined.
 
         Parameters
         ----------
@@ -111,34 +112,34 @@ class SimpleIntensifier(AbstractRacer):
 
         # If The incumbent is None we use the challenger
         if not incumbent:
-            self.logger.info(
-                "First run, no incumbent provided; challenger is assumed to be the incumbent"
-            )
+            self.logger.info("First run, no incumbent provided; challenger is assumed to be the incumbent")
             incumbent = run_info.config
 
         self.num_run += 1
 
-        incumbent = self._compare_configs(challenger=run_info.config,
-                                          incumbent=incumbent,
-                                          run_history=run_history,
-                                          log_traj=log_traj)
+        incumbent = self._compare_configs(
+            challenger=run_info.config,
+            incumbent=incumbent,
+            run_history=run_history,
+            log_traj=log_traj,
+        )
         # get incumbent cost
         inc_perf = run_history.get_cost(incumbent)
 
         return incumbent, inc_perf
 
-    def get_next_run(self,
-                     challengers: typing.Optional[typing.List[Configuration]],
-                     incumbent: Configuration,
-                     chooser: typing.Optional[EPMChooser],
-                     run_history: RunHistory,
-                     repeat_configs: bool = True,
-                     num_workers: int = 1,
-                     ) -> typing.Tuple[RunInfoIntent, RunInfo]:
-        """
-        Selects which challenger to be used. As in a traditional BO loop,
-        we sample from the EPM, which is the next configuration based on
-        the acquisition function. The input data is read from the runhistory.
+    def get_next_run(
+        self,
+        challengers: typing.Optional[typing.List[Configuration]],
+        incumbent: Configuration,
+        chooser: typing.Optional[EPMChooser],
+        run_history: RunHistory,
+        repeat_configs: bool = True,
+        num_workers: int = 1,
+    ) -> typing.Tuple[RunInfoIntent, RunInfo]:
+        """Selects which challenger to be used. As in a traditional BO loop, we sample from the EPM,
+        which is the next configuration based on the acquisition function. The input data is read
+        from the runhistory.
 
         Parameters
         ----------
@@ -163,12 +164,14 @@ class SimpleIntensifier(AbstractRacer):
         run_info: RunInfo
             An object that encapsulates the minimum information to
             evaluate a configuration
-         """
+        """
         # We always sample from the configs provided or the EPM
-        challenger = self._next_challenger(challengers=challengers,
-                                           chooser=chooser,
-                                           run_history=run_history,
-                                           repeat_configs=repeat_configs)
+        challenger = self._next_challenger(
+            challengers=challengers,
+            chooser=chooser,
+            run_history=run_history,
+            repeat_configs=repeat_configs,
+        )
 
         # Run tracker is a dictionary whose values indicate if a run has been
         # processed. If a value in this dict is false, it means that a worker
