@@ -1,13 +1,27 @@
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
+
 import collections
-from enum import Enum
 import json
-import typing
+from enum import Enum
 
 import numpy as np
 
 from smac.configspace import Configuration, ConfigurationSpace
 from smac.tae import StatusType
 from smac.utils.logging import PickableLoggerAdapter
+from smac.utils.multi_objective import normalize_costs
 
 __author__ = "Marius Lindauer"
 __copyright__ = "Copyright 2015, ML4AAD"
@@ -18,77 +32,90 @@ __version__ = "0.0.1"
 
 
 # NOTE class instead of collection to have a default value for budget in RunKey
-class RunKey(collections.namedtuple('RunKey', ['config_id', 'instance_id', 'seed', 'budget'])):
+class RunKey(collections.namedtuple("RunKey", ["config_id", "instance_id", "seed", "budget"])):
     __slots__ = ()
 
     def __new__(
         cls,  # No type annotation because the 1st argument for a namedtuble is always the class type,
-              # see https://docs.python.org/3/reference/datamodel.html#object.__new__
+        # see https://docs.python.org/3/reference/datamodel.html#object.__new__
         config_id: int,
-        instance_id: typing.Optional[str],
-        seed: typing.Optional[int],
+        instance_id: Optional[str],
+        seed: Optional[int],
         budget: float = 0.0,
-    ) -> 'RunKey':
+    ) -> "RunKey":
+        """Creates a new `RunKey` instance."""
         return super().__new__(cls, config_id, instance_id, seed, budget)
 
 
 # NOTE class instead of collection to have a default value for budget/source_id in RunInfo
 class RunInfo(
     collections.namedtuple(
-        'RunInfo',
-        ['config', 'instance', 'instance_specific', 'seed', 'cutoff', 'capped', 'budget', 'source_id']
+        "RunInfo",
+        [
+            "config",
+            "instance",
+            "instance_specific",
+            "seed",
+            "cutoff",
+            "capped",
+            "budget",
+            "source_id",
+        ],
     )
 ):
     __slots__ = ()
 
     def __new__(
         cls,  # No type annotation because the 1st argument for a namedtuble is always the class type,
-              # see https://docs.python.org/3/reference/datamodel.html#object.__new__
+        # see https://docs.python.org/3/reference/datamodel.html#object.__new__
         config: Configuration,
-        instance: typing.Optional[str],
+        instance: Optional[str],
         instance_specific: str,
         seed: int,
-        cutoff: typing.Optional[float],
+        cutoff: Optional[float],
         capped: bool,
         budget: float = 0.0,
         # In the context of parallel runs, one will have multiple suppliers of
         # configurations. source_id is a new mechanism to track what entity launched
         # this configuration
         source_id: int = 0,
-    ) -> 'RunInfo':
-        return super().__new__(cls, config, instance, instance_specific, seed,
-                               cutoff, capped, budget, source_id)
+    ) -> "RunInfo":
+        """Creates a new `RunInfo` instance."""
+        return super().__new__(
+            cls,
+            config,
+            instance,
+            instance_specific,
+            seed,
+            cutoff,
+            capped,
+            budget,
+            source_id,
+        )
 
 
-InstSeedKey = collections.namedtuple(
-    'InstSeedKey', ['instance', 'seed'])
+InstSeedKey = collections.namedtuple("InstSeedKey", ["instance", "seed"])
 
+InstSeedBudgetKey = collections.namedtuple("InstSeedBudgetKey", ["instance", "seed", "budget"])
 
-InstSeedBudgetKey = collections.namedtuple(
-    'InstSeedBudgetKey', ['instance', 'seed', 'budget'])
-
-
-RunValue = collections.namedtuple(
-    'RunValue', ['cost', 'time', 'status', 'starttime', 'endtime', 'additional_info'])
+RunValue = collections.namedtuple("RunValue", ["cost", "time", "status", "starttime", "endtime", "additional_info"])
 
 
 class EnumEncoder(json.JSONEncoder):
-    """Custom encoder for enum-serialization
-    (implemented for StatusType from tae).
-    Using encoder implied using object_hook as defined in StatusType
-    to deserialize from json.
+    """Custom encoder for enum-serialization (implemented for StatusType from tae).
+
+    Using encoder implied using object_hook as defined in StatusType to deserialize from json.
     """
 
-    def default(self, obj: object) -> typing.Any:
+    def default(self, obj: object) -> Any:
+        """Returns the default encoding of the passed object."""
         if isinstance(obj, StatusType):
             return {"__enum__": str(obj)}
         return json.JSONEncoder.default(self, obj)
 
 
 class DataOrigin(Enum):
-
-    """
-    Definition of how data in the runhistory is used.
+    """Definition of how data in the runhistory is used.
 
     * ``INTERNAL``: internal data which was gathered during the current
       optimization run. It will be saved to disk, used for building EPMs and
@@ -102,13 +129,13 @@ class DataOrigin(Enum):
        same instance features can still provide useful information. Will not be
        saved to disk and only used for EPM building.
     """
+
     INTERNAL = 1
     EXTERNAL_SAME_INSTANCES = 2
     EXTERNAL_DIFFERENT_INSTANCES = 3
 
 
-class RunHistory(object):
-
+class RunHistory(Mapping[RunKey, RunValue]):
     """Container for target algorithm run information.
 
     Most importantly, the runhistory contains an efficient mapping from each evaluated configuration to the
@@ -125,6 +152,13 @@ class RunHistory(object):
     ----
     Guaranteed to be picklable.
 
+    Parameters
+    ----------
+    overwrite_existing_runs : bool (default=False)
+        If set to ``True`` and a run of a configuration on an instance-budget-seed-pair already exists,
+        it is overwritten. Allows to overwrites old results if pairs of algorithm-instance-seed were measured
+        multiple times
+
     Attributes
     ----------
     data : collections.OrderedDict()
@@ -135,83 +169,67 @@ class RunHistory(object):
         Maps id -> config
     num_runs_per_config : dict
         Maps config_id -> number of runs
-
-    Parameters
-    ----------
-    overwrite_existing_runs : bool (default=True)
-        If set to ``True`` and a run of a configuration on an instance-budget-seed-pair already exists,
-        it is overwritten.
     """
 
     def __init__(
         self,
-        overwrite_existing_runs: bool = False
+        overwrite_existing_runs: bool = False,
     ) -> None:
-        """Constructor
-
-        Parameters
-        ----------
-        overwrite_existing_runs: bool
-            allows to overwrites old results if pairs of
-            algorithm-instance-seed were measured
-            multiple times
-        """
-        self.logger = PickableLoggerAdapter(
-            self.__module__ + "." + self.__class__.__name__
-        )
+        self.logger = PickableLoggerAdapter(self.__module__ + "." + self.__class__.__name__)
 
         # By having the data in a deterministic order we can do useful tests
         # when we serialize the data and can assume it's still in the same
         # order as it was added.
-        self.data = collections.OrderedDict()  # type: typing.Dict[RunKey, RunValue]
+        self.data = collections.OrderedDict()  # type: Dict[RunKey, RunValue]
 
         # for fast access, we have also an unordered data structure
         # to get all instance seed pairs of a configuration.
         # This does not include capped runs.
-        self._configid_to_inst_seed_budget = {}  # type: typing.Dict[int, typing.Dict[InstSeedKey, typing.List[float]]]
+        self._configid_to_inst_seed_budget = {}  # type: Dict[int, Dict[InstSeedKey, List[float]]]
 
-        self.config_ids = {}  # type: typing.Dict[Configuration, int]
-        self.ids_config = {}  # type: typing.Dict[int, Configuration]
+        self.config_ids = {}  # type: Dict[Configuration, int]
+        self.ids_config = {}  # type: Dict[int, Configuration]
         self._n_id = 0
 
         # Stores cost for each configuration ID
-        self._cost_per_config = {}  # type: typing.Dict[int, float]
+        self._cost_per_config = {}  # type: Dict[int, float]
         # Stores min cost across all budgets for each configuration ID
-        self._min_cost_per_config = {}  # type: typing.Dict[int, float]
+        self._min_cost_per_config = {}  # type: Dict[int, float]
         # runs_per_config maps the configuration ID to the number of runs for that configuration
         # and is necessary for computing the moving average
-        self.num_runs_per_config = {}  # type: typing.Dict[int, int]
+        self.num_runs_per_config = {}  # type: Dict[int, int]
 
         # Store whether a datapoint is "external", which means it was read from
         # a JSON file. Can be chosen to not be written to disk
-        self.external = {}  # type: typing.Dict[RunKey, DataOrigin]
+        self.external = {}  # type: Dict[RunKey, DataOrigin]
 
         self.overwrite_existing_runs = overwrite_existing_runs
+        self.num_obj = -1  # type: int
+        self.objective_bounds = []  # type: List[Tuple[float, float]]
 
     def add(
         self,
         config: Configuration,
-        cost: float,
+        cost: Union[int, float, list, np.ndarray],
         time: float,
         status: StatusType,
-        instance_id: typing.Optional[str] = None,
-        seed: typing.Optional[int] = None,
+        instance_id: Optional[str] = None,
+        seed: Optional[int] = None,
         budget: float = 0.0,
         starttime: float = 0.0,
         endtime: float = 0.0,
-        additional_info: typing.Optional[typing.Dict] = None,
+        additional_info: Optional[Dict] = None,
         origin: DataOrigin = DataOrigin.INTERNAL,
         force_update: bool = False,
     ) -> None:
-        """Adds a data of a new target algorithm (TA) run;
-        it will update data if the same key values are used
-        (config, instance_id, seed)
+        """Adds a data of a new target algorithm (TA) run; it will update data if the same key
+        values are used (config, instance_id, seed)
 
         Parameters
         ----------
             config : dict (or other type -- depending on config space module)
                 Parameter configuration
-            cost: float
+            cost: Union[int, float, list, np.ndarray]
                 Cost of TA run (will be minimized)
             time: float
                 Runtime of TA run
@@ -235,40 +253,52 @@ class RunHistory(object):
             force_update: bool (default: False)
                 Forces the addition of a config to the history
         """
-
         if config is None:
-            raise TypeError('Configuration to add to the runhistory must not be None')
+            raise TypeError("Configuration to add to the runhistory must not be None")
         elif not isinstance(config, Configuration):
             raise TypeError(
-                'Configuration to add to the runhistory is not of type Configuration, but %s' % type(config)
+                "Configuration to add to the runhistory is not of type Configuration, but %s" % type(config)
             )
 
+        # Squeeze is important to reduce arrays with one element
+        # to scalars.
+        cost = np.asarray(cost).squeeze()
+
         # Get the config id
         config_id_tmp = self.config_ids.get(config)
         if config_id_tmp is None:
             self._n_id += 1
             self.config_ids[config] = self._n_id
-            config_id = typing.cast(int, self.config_ids.get(config))
+            config_id = cast(int, self.config_ids.get(config))
             self.ids_config[self._n_id] = config
         else:
-            config_id = typing.cast(int, config_id_tmp)
+            config_id = cast(int, config_id_tmp)
+
+        if self.num_obj == -1:
+            self.num_obj = np.size(cost)
+        else:
+            if np.size(cost) != self.num_obj:
+                raise ValueError(
+                    f"Cost is not of the same length ({np.size(cost)}) as the number " f"of objectives ({self.num_obj})"
+                )
 
         k = RunKey(config_id, instance_id, seed, budget)
-        v = RunValue(cost, time, status, starttime, endtime, additional_info)
+        v = RunValue(cost.tolist(), time, status, starttime, endtime, additional_info)
+
         # Construct keys and values for the data dictionary
         for key, value in (
-            ('config', config.get_dictionary()),
-            ('config_id', config_id),
-            ('instance_id', instance_id),
-            ('seed', seed),
-            ('budget', budget),
-            ('cost', cost),
-            ('time', time),
-            ('status', status),
-            ('starttime', starttime),
-            ('endtime', endtime),
-            ('additional_info', additional_info),
-            ('origin', config.origin),
+            ("config", config.get_dictionary()),
+            ("config_id", config_id),
+            ("instance_id", instance_id),
+            ("seed", seed),
+            ("budget", budget),
+            ("cost", cost.tolist()),
+            ("time", time),
+            ("status", status),
+            ("starttime", starttime),
+            ("endtime", endtime),
+            ("additional_info", additional_info),
+            ("origin", config.origin),
         ):
             self._check_json_serializable(key, value, EnumEncoder, k, v)
 
@@ -286,10 +316,10 @@ class RunHistory(object):
     def _check_json_serializable(
         self,
         key: str,
-        obj: typing.Any,
-        encoder: typing.Type[json.JSONEncoder],
+        obj: Any,
+        encoder: Type[json.JSONEncoder],
         runkey: RunKey,
-        runvalue: RunValue
+        runvalue: RunValue,
     ) -> None:
         try:
             json.dumps(obj, cls=encoder)
@@ -299,20 +329,44 @@ class RunHistory(object):
                 "please see the error above.\nRunKey: %s\nRunValue %s" % (key, str(obj), type(obj), runkey, runvalue)
             ) from e
 
-    def _add(self, k: RunKey, v: RunValue, status: StatusType,
-             origin: DataOrigin) -> None:
-        """Actual function to add new entry to data structures
+    def _update_objective_bounds(self) -> None:
+        """Update the objective bounds based on the data in the runhistory."""
+        all_costs = []
+        for (costs, _, status, _, _, _) in self.data.values():
+            if status == StatusType.SUCCESS:
+                if not isinstance(costs, Iterable):
+                    costs = [costs]
 
-        TODO
+                assert len(costs) == self.num_obj
+                all_costs.append(costs)
 
-        """
+        all_costs = np.array(all_costs, dtype=float)  # type: ignore[assignment]
+
+        if len(all_costs) == 0:
+            self.objective_bounds = [(np.inf, -np.inf)] * self.num_obj
+            return
+
+        min_values = np.min(all_costs, axis=0)
+        max_values = np.max(all_costs, axis=0)
+
+        self.objective_bounds = []
+        for min_v, max_v in zip(min_values, max_values):
+            self.objective_bounds += [(min_v, max_v)]
+
+    def _add(self, k: RunKey, v: RunValue, status: StatusType, origin: DataOrigin) -> None:
+        """Actual function to add new entry to data structures."""
         self.data[k] = v
         self.external[k] = origin
 
+        # Update objective bounds
+        self._update_objective_bounds()
+
         # Capped data is added above
         # Do not register the cost until the run has completed
-        if origin in (DataOrigin.INTERNAL, DataOrigin.EXTERNAL_SAME_INSTANCES) \
-                and status not in [StatusType.CAPPED, StatusType.RUNNING]:
+        if origin in (
+            DataOrigin.INTERNAL,
+            DataOrigin.EXTERNAL_SAME_INSTANCES,
+        ) and status not in [StatusType.CAPPED, StatusType.RUNNING]:
             # also add to fast data structure
             is_k = InstSeedKey(k.instance_id, k.seed)
             self._configid_to_inst_seed_budget[k.config_id] = self._configid_to_inst_seed_budget.get(k.config_id, {})
@@ -333,11 +387,11 @@ class RunHistory(object):
                 self.update_cost(config=self.ids_config[k.config_id])
                 if k.budget > 0:
                     if self.num_runs_per_config[k.config_id] != 1:  # This is updated in update_cost
-                        raise ValueError('This should not happen!')
+                        raise ValueError("This should not happen!")
 
     def update_cost(self, config: Configuration) -> None:
-        """Store the performance of a configuration across the instances in
-        self.cost_per_config and also updates self.runs_per_config;
+        """Stores the performance of a configuration across the instances in self.cost_per_config
+        and also updates self.runs_per_config;
 
         Note
         ----
@@ -357,9 +411,8 @@ class RunHistory(object):
         all_inst_seed_budgets = list(dict.fromkeys(self.get_runs_for_config(config, only_max_observed_budget=False)))
         self._min_cost_per_config[config_id] = self.min_cost(config, all_inst_seed_budgets)
 
-    def incremental_update_cost(self, config: Configuration, cost: float) -> None:
-        """Incrementally updates the performance of a configuration by using a
-        moving average;
+    def incremental_update_cost(self, config: Configuration, cost: Union[np.ndarray, list, float, int]) -> None:
+        """Incrementally updates the performance of a configuration by using a moving average.
 
         Parameters
         ----------
@@ -368,17 +421,19 @@ class RunHistory(object):
         cost: float
             cost of new run of config
         """
-
         config_id = self.config_ids[config]
         n_runs = self.num_runs_per_config.get(config_id, 0)
-        old_cost = self._cost_per_config.get(config_id, 0.)
-        self._cost_per_config[config_id] = ((old_cost * n_runs) + cost) / (n_runs + 1)
+        old_cost = self._cost_per_config.get(config_id, 0.0)
+
+        if self.num_obj > 1:
+            cost = self.average_cost(config)
+
+        self._cost_per_config[config_id] = ((old_cost * n_runs) + cost) / (n_runs + 1)  # type: ignore
         self.num_runs_per_config[config_id] = n_runs + 1
 
     def get_cost(self, config: Configuration) -> float:
-        """Returns empirical cost for a configuration.
-
-        See the class docstring for how the costs are computed. The costs are not re-computed, but are read from cache.
+        """Returns empirical cost for a configuration. See the class docstring for how the costs are
+        computed. The costs are not re-computed, but are read from cache.
 
         Parameters
         ----------
@@ -392,8 +447,7 @@ class RunHistory(object):
         config_id = self.config_ids.get(config)
         return self._cost_per_config.get(config_id, np.nan)  # type: ignore[arg-type] # noqa F821
 
-    def get_runs_for_config(self,
-                            config: Configuration, only_max_observed_budget: bool) -> typing.List[InstSeedBudgetKey]:
+    def get_runs_for_config(self, config: Configuration, only_max_observed_budget: bool) -> List[InstSeedBudgetKey]:
         """Return all runs (instance seed pairs) for a configuration.
 
         Note
@@ -422,8 +476,8 @@ class RunHistory(object):
         rval = [InstSeedBudgetKey(k.instance, k.seed, budget) for k, v in runs.items() for budget in v]
         return rval
 
-    def get_all_configs(self) -> typing.List[Configuration]:
-        """Return all configurations in this RunHistory object
+    def get_all_configs(self) -> List[Configuration]:
+        """Return all configurations in this RunHistory object.
 
         Returns
         -------
@@ -433,10 +487,9 @@ class RunHistory(object):
 
     def get_all_configs_per_budget(
         self,
-        budget_subset: typing.Optional[typing.List] = None,
-    ) -> typing.List[Configuration]:
-        """
-        Return all configs in this RunHistory object that have been run on one of these budgets
+        budget_subset: Optional[List] = None,
+    ) -> List[Configuration]:
+        """Return all configs in this RunHistory object that have been run on one of these budgets.
 
         Parameters
         ----------
@@ -457,7 +510,8 @@ class RunHistory(object):
     def get_min_cost(self, config: Configuration) -> float:
         """Returns the lowest empirical cost for a configuration, across all runs (budgets)
 
-        See the class docstring for how the costs are computed. The costs are not re-computed, but are read from cache.
+        See the class docstring for how the costs are computed. The costs are not re-computed,
+        but are read from cache.
 
         Parameters
         ----------
@@ -483,38 +537,51 @@ class RunHistory(object):
         return len(self.data) == 0
 
     def save_json(self, fn: str = "runhistory.json", save_external: bool = False) -> None:
-        """
-        saves runhistory on disk
+        """Saves runhistory on disk.
 
         Parameters
         ----------
         fn : str
-            file name
+            file name.
         save_external : bool
             Whether to save external data in the runhistory file.
         """
-        data = [([int(k.config_id),
-                  str(k.instance_id) if k.instance_id is not None else None,
-                  int(k.seed),
-                  float(k.budget) if k[3] is not None else 0], list(v))
-                for k, v in self.data.items()
-                if save_external or self.external[k] == DataOrigin.INTERNAL]
+        data = [
+            (
+                [
+                    int(k.config_id),
+                    str(k.instance_id) if k.instance_id is not None else None,
+                    int(k.seed),
+                    float(k.budget) if k[3] is not None else 0,
+                ],
+                [v.cost, v.time, v.status, v.starttime, v.endtime, v.additional_info],
+            )
+            for k, v in self.data.items()
+            if save_external or self.external[k] == DataOrigin.INTERNAL
+        ]
         config_ids_to_serialize = set([entry[0][0] for entry in data])
-        configs = {id_: conf.get_dictionary()
-                   for id_, conf in self.ids_config.items()
-                   if id_ in config_ids_to_serialize}
-        config_origins = {id_: conf.origin
-                          for id_, conf in self.ids_config.items()
-                          if (id_ in config_ids_to_serialize and conf.origin is not None)}
+        configs = {
+            id_: conf.get_dictionary() for id_, conf in self.ids_config.items() if id_ in config_ids_to_serialize
+        }
+        config_origins = {
+            id_: conf.origin
+            for id_, conf in self.ids_config.items()
+            if (id_ in config_ids_to_serialize and conf.origin is not None)
+        }
 
         with open(fn, "w") as fp:
-            json.dump({"data": data,
-                       "config_origins": config_origins,
-                       "configs": configs}, fp, cls=EnumEncoder, indent=2)
+            json.dump(
+                {"data": data, "config_origins": config_origins, "configs": configs},
+                fp,
+                cls=EnumEncoder,
+                indent=2,
+            )
 
     def load_json(self, fn: str, cs: ConfigurationSpace) -> None:
         """Load and runhistory in json representation from disk.
 
+        Warning
+        -------
         Overwrites current runhistory!
 
         Parameters
@@ -529,8 +596,7 @@ class RunHistory(object):
                 all_data = json.load(fp, object_hook=StatusType.enum_hook)
         except Exception as e:
             self.logger.warning(
-                'Encountered exception %s while reading runhistory from %s. '
-                'Not adding any runs!',
+                "Encountered exception %s while reading runhistory from %s. " "Not adding any runs!",
                 e,
                 fn,
             )
@@ -539,27 +605,39 @@ class RunHistory(object):
         config_origins = all_data.get("config_origins", {})
 
         self.ids_config = {
-            int(id_): Configuration(
-                cs, values=values, origin=config_origins.get(id_, None)
-            ) for id_, values in all_data["configs"].items()
+            int(id_): Configuration(cs, values=values, origin=config_origins.get(id_, None))
+            for id_, values in all_data["configs"].items()
         }
 
         self.config_ids = {config: id_ for id_, config in self.ids_config.items()}
-
         self._n_id = len(self.config_ids)
 
         # important to use add method to use all data structure correctly
         for k, v in all_data["data"]:
-            self.add(config=self.ids_config[int(k[0])],
-                     cost=float(v[0]),
-                     time=float(v[1]),
-                     status=StatusType(v[2]),
-                     instance_id=k[1],
-                     seed=int(k[2]),
-                     budget=float(k[3]) if len(k) == 4 else 0,
-                     starttime=v[3],
-                     endtime=v[4],
-                     additional_info=v[5])
+            # Set num_obj first
+            if self.num_obj == -1:
+                if isinstance(v[0], float) or isinstance(v[0], int):
+                    self.num_obj = 1
+                else:
+                    self.num_obj = len(np.asarray(list(map(float, v[0]))))
+
+            if self.num_obj == 1:
+                cost: Union[np.ndarray, float] = float(v[0])
+            else:
+                cost: Union[np.ndarray, float] = np.asarray(list(map(float, v[0])))  # type: ignore[no-redef]
+
+            self.add(
+                config=self.ids_config[int(k[0])],
+                cost=cost,
+                time=float(v[1]),
+                status=StatusType(v[2]),
+                instance_id=k[1],
+                seed=int(k[2]),
+                budget=float(k[3]) if len(k) == 4 else 0,
+                starttime=v[3],
+                endtime=v[4],
+                additional_info=v[5],
+            )
 
     def update_from_json(
         self,
@@ -567,7 +645,7 @@ class RunHistory(object):
         cs: ConfigurationSpace,
         origin: DataOrigin = DataOrigin.EXTERNAL_SAME_INSTANCES,
     ) -> None:
-        """Update the current runhistory by adding new runs from a json file.
+        """Updates the current runhistory by adding new runs from a json file.
 
         Parameters
         ----------
@@ -584,10 +662,10 @@ class RunHistory(object):
 
     def update(
         self,
-        runhistory: 'RunHistory',
+        runhistory: "RunHistory",
         origin: DataOrigin = DataOrigin.EXTERNAL_SAME_INSTANCES,
     ) -> None:
-        """Update the current runhistory by adding new runs from a RunHistory.
+        """Updates the current runhistory by adding new runs from a RunHistory.
 
         Parameters
         ----------
@@ -598,7 +676,6 @@ class RunHistory(object):
             added to the internal data structure self._configid_to_inst_seed_budget
             and be available :meth:`through get_runs_for_config`.
         """
-
         # Configurations might be already known, but by a different ID. This
         # does not matter here because the add() method handles this
         # correctly by assigning an ID to unknown configurations and re-using
@@ -607,22 +684,31 @@ class RunHistory(object):
             config_id, instance_id, seed, budget = key
             cost, time, status, start, end, additional_info = value
             config = runhistory.ids_config[config_id]
-            self.add(config=config, cost=cost, time=time,
-                     status=status, instance_id=instance_id, starttime=start, endtime=end,
-                     seed=seed, budget=budget, additional_info=additional_info,
-                     origin=origin)
+            self.add(
+                config=config,
+                cost=cost,
+                time=time,
+                status=status,
+                instance_id=instance_id,
+                starttime=start,
+                endtime=end,
+                seed=seed,
+                budget=budget,
+                additional_info=additional_info,
+                origin=origin,
+            )
 
     def _cost(
         self,
         config: Configuration,
-        instance_seed_budget_keys: typing.Optional[typing.Iterable[InstSeedBudgetKey]] = None,
-    ) -> typing.List[float]:
+        instance_seed_budget_keys: Optional[Iterable[InstSeedBudgetKey]] = None,
+    ) -> List[np.ndarray]:
         """Return array of all costs for the given config for further calculations.
 
         Parameters
         ----------
         config : Configuration
-            Configuration to calculate objective for
+            Configuration to calculate objective for.
         instance_seed_budget_keys : list, optional (default=None)
             List of tuples of instance-seeds-budget keys. If None, the run_history is
             queried for all runs of the given configuration.
@@ -630,7 +716,7 @@ class RunHistory(object):
         Returns
         -------
         Costs: list
-            Array of all costs
+            Array of all costs.
         """
         try:
             id_ = self.config_ids[config]
@@ -644,32 +730,36 @@ class RunHistory(object):
         for i, r, b in instance_seed_budget_keys:
             k = RunKey(id_, i, r, b)
             costs.append(self.data[k].cost)
+
         return costs
 
     def average_cost(
         self,
         config: Configuration,
-        instance_seed_budget_keys: typing.Optional[typing.Iterable[InstSeedBudgetKey]] = None,
+        instance_seed_budget_keys: Optional[Iterable[InstSeedBudgetKey]] = None,
     ) -> float:
-        """Return the average cost of a configuration.
-
-        This is the mean of costs of all instance-seed pairs.
+        """Return the average cost of a configuration. This is the mean of costs of all instance-
+        seed pairs.
 
         Parameters
         ----------
         config : Configuration
-            Configuration to calculate objective for
+            Configuration to calculate objective for.
         instance_seed_budget_keys : list, optional (default=None)
             List of tuples of instance-seeds-budget keys. If None, the run_history is
             queried for all runs of the given configuration.
 
         Returns
-        ----------
+        -------
         Cost: float
             Average cost
         """
         costs = self._cost(config, instance_seed_budget_keys)
         if costs:
+            if self.num_obj > 1:
+                # Normalize costs
+                costs = normalize_costs(costs, self.objective_bounds)  # type: ignore[assignment]
+
             return float(np.mean(costs))
 
         return np.nan
@@ -677,58 +767,70 @@ class RunHistory(object):
     def sum_cost(
         self,
         config: Configuration,
-        instance_seed_budget_keys: typing.Optional[typing.Iterable[InstSeedBudgetKey]] = None,
+        instance_seed_budget_keys: Optional[Iterable[InstSeedBudgetKey]] = None,
     ) -> float:
-        """Return the sum of costs of a configuration.
-
-        This is the sum of costs of all instance-seed pairs.
+        """Return the sum of costs of a configuration. This is the sum of costs of all instance-seed
+        pairs.
 
         Parameters
         ----------
         config : Configuration
-            Configuration to calculate objective for
+            Configuration to calculate objective for.
         instance_seed_budget_keys : list, optional (default=None)
             List of tuples of instance-seeds-budget keys. If None, the run_history is
             queried for all runs of the given configuration.
 
         Returns
-        ----------
+        -------
         sum_cost: float
-            Sum of costs of config
+            Sum of costs of config.
         """
-        return float(np.sum(self._cost(config, instance_seed_budget_keys)))
+        costs = self._cost(config, instance_seed_budget_keys)
+        if costs:
+            if self.num_obj > 1:
+                # Normalize costs
+                costs = normalize_costs(costs, self.objective_bounds)  # type: ignore[assignment]
+                costs = np.mean(costs, axis=1)
+
+        return float(np.sum(costs))
 
     def min_cost(
         self,
         config: Configuration,
-        instance_seed_budget_keys: typing.Optional[typing.Iterable[InstSeedBudgetKey]] = None,
+        instance_seed_budget_keys: Optional[Iterable[InstSeedBudgetKey]] = None,
     ) -> float:
-        """Return the minimum cost of a configuration
+        """Return the minimum cost of a configuration.
 
         This is the minimum cost of all instance-seed pairs.
+        Warning: In the case of multi-fidelity, the minimum cost per objectives is returned.
 
         Parameters
         ----------
         config : Configuration
-            Configuration to calculate objective for
+            Configuration to calculate objective for.
         instance_seed_budget_keys : list, optional (default=None)
             List of tuples of instance-seeds-budget keys. If None, the run_history is
             queried for all runs of the given configuration.
 
         Returns
-        ----------
+        -------
         min_cost: float
-            minimum cost of config
+            minimum cost of config.
         """
         costs = self._cost(config, instance_seed_budget_keys)
         if costs:
+            if self.num_obj > 1:
+                # Normalize costs
+                costs = normalize_costs(costs, self.objective_bounds)  # type: ignore[assignment]
+                costs = np.mean(costs, axis=1)
+
             return float(np.min(costs))
 
         return np.nan
 
-    def compute_all_costs(self, instances: typing.Optional[typing.List[str]] = None) -> None:
-        """Computes the cost of all configurations from scratch and overwrites
-        self.cost_perf_config and self.runs_per_config accordingly;
+    def compute_all_costs(self, instances: Optional[List[str]] = None) -> None:
+        """Computes the cost of all configurations from scratch and overwrites self.cost_perf_config
+        and self.runs_per_config accordingly.
 
         Note
         ----
@@ -736,8 +838,8 @@ class RunHistory(object):
 
         Parameters
         ----------
-        instances: typing.List[str]
-            list of instances; if given, cost is only computed wrt to this instance set
+        instances: List[str]
+            List of instances; if given, cost is only computed wrt to this instance set.
         """
         self._cost_per_config = {}
         self.num_runs_per_config = {}
@@ -745,21 +847,16 @@ class RunHistory(object):
             # removing duplicates while keeping the order
             inst_seed_budgets = list(dict.fromkeys(self.get_runs_for_config(config, only_max_observed_budget=True)))
             if instances is not None:
-                inst_seed_budgets = list(
-                    filter(
-                        lambda x: x.instance in typing.cast(typing.List, instances), inst_seed_budgets
-                    )
-                )
+                inst_seed_budgets = list(filter(lambda x: x.instance in cast(List, instances), inst_seed_budgets))
 
             if inst_seed_budgets:  # can be empty if never saw any runs on <instances>
                 self._cost_per_config[config_id] = self.average_cost(config, inst_seed_budgets)
                 self._min_cost_per_config[config_id] = self.min_cost(config, inst_seed_budgets)
                 self.num_runs_per_config[config_id] = len(inst_seed_budgets)
 
-    def get_instance_costs_for_config(self, config: Configuration) -> typing.Dict[str, typing.List[float]]:
-        """ Returns the average cost per instance (across seeds) for a configuration
-
-        If the runhistory contains budgets, only the highest budget for a configuration is returned.
+    def get_instance_costs_for_config(self, config: Configuration) -> Dict[str, List[float]]:
+        """Returns the average cost per instance (across seeds) for a configuration. If the
+        runhistory contains budgets, only the highest budget for a configuration is returned.
 
         Note
         ----
@@ -772,10 +869,10 @@ class RunHistory(object):
 
         Returns
         -------
-        cost_per_inst: dict<instance name<str>, cost<float>>
+        cost_per_inst: Dict<instance name<str>, cost<float>>
         """
         runs_ = self.get_runs_for_config(config, only_max_observed_budget=True)
-        cost_per_inst = {}  # type: typing.Dict[str, typing.List[float]]
+        cost_per_inst = {}  # type: Dict[str, List[float]]
         for inst, seed, budget in runs_:
             cost_per_inst[inst] = cost_per_inst.get(inst, [])
             rkey = RunKey(self.config_ids[config], inst, seed, budget)
@@ -783,3 +880,22 @@ class RunHistory(object):
             cost_per_inst[inst].append(vkey.cost)
         cost_per_inst = dict([(inst, np.mean(costs)) for inst, costs in cost_per_inst.items()])
         return cost_per_inst
+
+    def __contains__(self, k: object) -> bool:
+        """Dictionary semantics for `k in runhistory`"""
+        return k in self.data
+
+    def __getitem__(self, k: RunKey) -> RunValue:
+        """Dictionary semantics for `v = runhistory[k]`"""
+        return self.data[k]
+
+    def __iter__(self) -> Iterator[RunKey]:
+        """Dictionary semantics for `for k in runhistory.keys()`, enables.
+
+        .items()
+        """
+        return iter(self.data.keys())
+
+    def __len__(self) -> int:
+        """Enables the `len(runhistory)`"""
+        return len(self.data)
