@@ -1,17 +1,18 @@
 from abc import ABC, abstractmethod
+from typing import Callable, Dict, List, Optional, Tuple, Union
+
 import math
 import time
 import traceback
-import typing
 
 import numpy as np
 
 from smac.configspace import Configuration
-from smac.utils.constants import MAXINT
-from smac.utils.logging import PickableLoggerAdapter
 from smac.runhistory.runhistory import RunInfo, RunValue
 from smac.stats.stats import Stats
 from smac.tae import StatusType
+from smac.utils.constants import MAXINT
+from smac.utils.logging import PickableLoggerAdapter
 
 __copyright__ = "Copyright 2021, AutoML.org Freiburg-Hannover"
 __license__ = "3-clause BSD"
@@ -45,70 +46,70 @@ class BaseRunner(ABC):
        example, the intensifier might not be able to select the next challenger
        until more results are available.
 
+    Parameters
+    ----------
+    ta : Union[List[str], Callable]
+        target algorithm
+    stats: Stats
+         stats object to collect statistics about runtime/additional info
+    multi_objectives: List[str]
+        names of the objectives, by default it is a single objective parameter "cost"
+    run_obj: str
+        run objective of SMAC
+    par_factor: int
+        penalization factor
+    cost_for_crash : float
+        cost that is used in case of crashed runs (including runs
+        that returned NaN or inf)
+    abort_on_first_run_crash: bool
+        if true and first run crashes, raise FirstRunCrashedException
+
+
+    Attributes
+    ----------
+    results
+    ta
+    stats
+    run_obj
+    par_factor
+    cost_for_crash
+    abort_on_first_run_crash
     """
 
     def __init__(
         self,
-        ta: typing.Union[typing.List[str], typing.Callable],
+        ta: Union[List[str], Callable],
         stats: Stats,
+        multi_objectives: List[str] = ["cost"],
         run_obj: str = "runtime",
         par_factor: int = 1,
-        cost_for_crash: float = float(MAXINT),
+        cost_for_crash: Union[float, List[float]] = float(MAXINT),
         abort_on_first_run_crash: bool = True,
     ):
-        """
-        Attributes
-        ----------
-        results
-        ta
-        stats
-        run_obj
-        par_factor
-        cost_for_crash
-        abort_first_run_crash
-
-        Parameters
-        ----------
-        ta : typing.Union[typing.List[str], typing.Callable]
-            target algorithm
-        stats: Stats
-             stats object to collect statistics about runtime/additional info
-        run_obj: str
-            run objective of SMAC
-        par_factor: int
-            penalization factor
-        cost_for_crash : float
-            cost that is used in case of crashed runs (including runs
-            that returned NaN or inf)
-        abort_on_first_run_crash: bool
-            if true and first run crashes, raise FirstRunCrashedException
-        """
-
         # The results is a FIFO structure, implemented via a list
         # (because the Queue lock is not pickable). Finished runs are
         # put in this list and collected via process_finished_runs
-        self.results = []  # type: typing.List[typing.Tuple[RunInfo, RunValue]]
+        self.results = []  # type: List[Tuple[RunInfo, RunValue]]
 
         # Below state the support for a Runner algorithm that
         # implements a ta
         self.ta = ta
         self.stats = stats
+        self.multi_objectives = multi_objectives
         self.run_obj = run_obj
         self.par_factor = par_factor
         self.cost_for_crash = cost_for_crash
         self.abort_on_first_run_crash = abort_on_first_run_crash
-        self.logger = PickableLoggerAdapter(
-            self.__module__ + '.' + self.__class__.__name__)
+        self.logger = PickableLoggerAdapter(self.__module__ + "." + self.__class__.__name__)
         self._supports_memory_limit = False
 
         super().__init__()
 
     @abstractmethod
     def submit_run(self, run_info: RunInfo) -> None:
-        """This function submits a configuration
-        embedded in a RunInfo object, and uses one of the workers
-        to produce a result (such result will eventually be available
-        on the self.results FIFO).
+        """This function submits a configuration embedded in a RunInfo object, and uses one of the
+        workers to produce a result (such result will eventually be available on the self.results
+        FIFO).
 
         This interface method will be called by SMBO, with the expectation
         that a function will be executed by a worker.
@@ -123,21 +124,24 @@ class BaseRunner(ABC):
         ----------
         run_info: RunInfo
             An object containing the configuration and the necessary data to run it
-
         """
         pass
 
     @abstractmethod
     def run(
-        self, config: Configuration,
+        self,
+        config: Configuration,
         instance: str,
-        cutoff: typing.Optional[float] = None,
+        cutoff: Optional[float] = None,
         seed: int = 12345,
-        budget: typing.Optional[float] = None,
+        budget: Optional[float] = None,
         instance_specific: str = "0",
-    ) -> typing.Tuple[StatusType, float, float, typing.Dict]:
-        """Runs target algorithm <self.ta> with configuration <config> on
-        instance <instance> with instance specifics <specifics> for at most
+    ) -> Tuple[StatusType, float, float, Dict]:
+        """Runs target algorithm <self.ta> with configuration <config> on instance <instance> with
+        instance specifics.
+
+        <specifics> for at most.
+
         <cutoff> seconds and random seed <seed>
 
         This method exemplifies how to defined the run() method
@@ -175,8 +179,8 @@ class BaseRunner(ABC):
     def run_wrapper(
         self,
         run_info: RunInfo,
-    ) -> typing.Tuple[RunInfo, RunValue]:
-        """Wrapper around run() to exec and check the execution of a given config file
+    ) -> Tuple[RunInfo, RunValue]:
+        """Wrapper around run() to exec and check the execution of a given config file.
 
         This function encapsulates common handling/processing, so that run() implementation
         is simplified.
@@ -219,43 +223,35 @@ class BaseRunner(ABC):
                 cutoff=cutoff,
                 seed=run_info.seed,
                 budget=run_info.budget,
-                instance_specific=run_info.instance_specific
+                instance_specific=run_info.instance_specific,
             )
         except Exception as e:
             status = StatusType.CRASHED
-            cost = self.cost_for_crash
+            cost = self.cost_for_crash  # type: ignore
             runtime = time.time() - start
 
             # Add context information to the error message
             exception_traceback = traceback.format_exc()
             error_message = repr(e)
-            additional_info = {
-                'traceback': exception_traceback,
-                'error': error_message
-            }
+            additional_info = {"traceback": exception_traceback, "error": error_message}
 
         end = time.time()
 
         if run_info.budget == 0 and status == StatusType.DONOTADVANCE:
-            raise ValueError(
-                "Cannot handle DONOTADVANCE state when using intensify or SH/HB on "
-                "instances."
-            )
+            raise ValueError("Cannot handle DONOTADVANCE state when using intensify or SH/HB on " "instances.")
 
         # Catch NaN or inf.
-        if (
-            self.run_obj == 'runtime' and not np.isfinite(runtime)
-            or self.run_obj == 'quality' and not np.isfinite(cost)
+        if (self.run_obj == "runtime" and not np.isfinite(runtime)) or (
+            self.run_obj == "quality" and not np.all(np.isfinite(cost))
         ):
             if self.logger:
-                self.logger.warning("Target Algorithm returned NaN or inf as {}. "
-                                    "Algorithm run is treated as CRASHED, cost "
-                                    "is set to {} for quality scenarios. "
-                                    "(Change value through \"cost_for_crash\""
-                                    "-option.)".format(
-                                        self.run_obj,
-                                        self.cost_for_crash)
-                                    )
+                self.logger.warning(
+                    "Target Algorithm returned NaN or inf as {}. "
+                    "Algorithm run is treated as CRASHED, cost "
+                    "is set to {} for quality scenarios. "
+                    '(Change value through "cost_for_crash"'
+                    "-option.)".format(self.run_obj, self.cost_for_crash)
+                )
             status = StatusType.CRASHED
 
         if self.run_obj == "runtime":
@@ -267,7 +263,8 @@ class BaseRunner(ABC):
                 self.logger.warning(
                     "Returned running time is larger "
                     "than {0} times the passed cutoff time. "
-                    "Clamping to {0} x cutoff.".format(self.par_factor))
+                    "Clamping to {0} x cutoff.".format(self.par_factor)
+                )
                 runtime = cutoff * self.par_factor
                 status = StatusType.TIMEOUT
             if status == StatusType.SUCCESS:
@@ -278,7 +275,7 @@ class BaseRunner(ABC):
                 status = StatusType.CAPPED
         else:
             if status == StatusType.CRASHED:
-                cost = self.cost_for_crash
+                cost = self.cost_for_crash  # type: ignore
 
         return run_info, RunValue(
             status=status,
@@ -286,16 +283,15 @@ class BaseRunner(ABC):
             time=runtime,
             additional_info=additional_info,
             starttime=start,
-            endtime=end
+            endtime=end,
         )
 
     @abstractmethod
-    def get_finished_runs(self) -> typing.List[typing.Tuple[RunInfo, RunValue]]:
-        """This method returns any finished configuration, and returns a list with
-        the results of exercising the configurations. This class keeps populating results
-        to self.results until a call to get_finished runs is done. In this case, the
-        self.results list is emptied and all RunValues produced by running run() are
-        returned.
+    def get_finished_runs(self) -> List[Tuple[RunInfo, RunValue]]:
+        """This method returns any finished configuration, and returns a list with the results of
+        exercising the configurations. This class keeps populating results to self.results until a
+        call to get_finished runs is done. In this case, the self.results list is emptied and all
+        RunValues produced by running run() are returned.
 
         Returns
         -------
@@ -307,22 +303,21 @@ class BaseRunner(ABC):
     @abstractmethod
     def wait(self) -> None:
         """SMBO/intensifier might need to wait for runs to finish before making a decision.
+
         This method waits until 1 run completes
         """
         pass
 
     @abstractmethod
     def pending_runs(self) -> bool:
-        """
-        Whether or not there are configs still running. Generally if the runner is serial,
-        launching a run instantly returns it's result. On parallel runners, there might
-        be pending configurations to complete.
+        """Whether or not there are configs still running.
+
+        Generally if the runner is serial, launching a run instantly returns it's result. On
+        parallel runners, there might be pending configurations to complete.
         """
         pass
 
     @abstractmethod
     def num_workers(self) -> int:
-        """
-        Return the active number of workers that will execute tae runs.
-        """
+        """Return the active number of workers that will execute tae runs."""
         pass
