@@ -6,7 +6,8 @@ __license__ = "3-clause BSD"
 
 
 def get_branin_config_space() -> ConfigurationSpace:
+    """Returns the branin configspace."""
     cs = ConfigurationSpace()
-    cs.add_hyperparameter(UniformFloatHyperparameter('x', -5, 10))
-    cs.add_hyperparameter(UniformFloatHyperparameter('y', 0, 15))
+    cs.add_hyperparameter(UniformFloatHyperparameter("x", -5, 10))
+    cs.add_hyperparameter(UniformFloatHyperparameter("y", 0, 15))
     return cs
